/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package br.com.flaviobarros.simuladordeinvestimentos.cenarios.controller;

import br.com.flaviobarros.simuladordeinvestimentos.investimentos.model.Investimento;
import br.com.flaviobarros.simuladordeinvestimentos.investimentos.model.InvestimentosComunsSemelhantes;
import br.com.flaviobarros.simuladordeinvestimentos.investimentos.controller.GerenciaArmazenamentoInvestimentos;
import br.com.flaviobarros.simuladordeinvestimentos.investimentos.indices.controller.GerenciaArmazenamentoIndices;
import br.com.flaviobarros.simuladordeinvestimentos.investimentos.indices.model.Indice;
import java.util.List;

/**
 *
 * @author flavio
 *
 */
public class GerenciadorAuxiliarCenariosDeSimulacao {

    /**
     * Retorna true se os investimentos e indices referentes aos investimentos cadastrados no banco de dados
     * estão corretamente preenchidos
     * (verifica assim se os cenários para os investimentos em questão já foram adequadamente definidos).
     * 
     * @param cenario1:         boolean indicando se o cenário em questão sera utilizado/verificado
     * @param cenario2:         boolean indicando se o cenário em questão sera utilizado/verificado
     * @param cenario3:         boolean indicando se o cenário em questão sera utilizado/verificado
     * @param cenario4:         boolean indicando se o cenário em questão sera utilizado/verificado
     * @param cenario5:         boolean indicando se o cenário em questão sera utilizado/verificado
     * @param cenario6:         boolean indicando se o cenário em questão sera utilizado/verificado
     * 
     * @return TRUE caso os cenários e índices analizados estejam corretamente definidos e FALSE caso contrário.
     */
    public boolean cenariosEstaoDefinidos(boolean cenario1, boolean cenario2, boolean cenario3, boolean cenario4, boolean cenario5, boolean cenario6){
        
        // obtendo os dados dos investimentos do banco de dados
        GerenciaArmazenamentoInvestimentos gerenciadorInvestimentos = new GerenciaArmazenamentoInvestimentos();
        List<Investimento> investimentos = gerenciadorInvestimentos.getRegistrosDeTodosOsInvestimentos();
        
        return cenariosEstaoDefinidos(investimentos, cenario1, cenario2, cenario3, cenario4, cenario5, cenario6);
        
    }
    
    
    /**
     * Retorna true se os investimentos e indices referentes aos cenários recebidos como parâmetros estão corretamente preenchidos
     * (verifica assim se os cenários para os investimentos em questão já foram adequadamente definidos).
     * ** Observação: caso algum dos indices necessários aínda não esteja cadastrado no banco esta função também realiza o cadastramento
     *    do índice.
     * 
     * @param investimentos:    Lista dos investimentos cadastrados
     * @param cenario1:         boolean indicando se o cenário em questão sera utilizado/verificado
     * @param cenario2:         boolean indicando se o cenário em questão sera utilizado/verificado
     * @param cenario3:         boolean indicando se o cenário em questão sera utilizado/verificado
     * @param cenario4:         boolean indicando se o cenário em questão sera utilizado/verificado
     * @param cenario5:         boolean indicando se o cenário em questão sera utilizado/verificado
     * @param cenario6:         boolean indicando se o cenário em questão sera utilizado/verificado
     * 
     * @return TRUE caso os cenários e índices analizados estejam corretamente definidos e FALSE caso contrário.
     */
    private boolean cenariosEstaoDefinidos(List<Investimento> investimentos, boolean cenario1, boolean cenario2, boolean cenario3, boolean cenario4, boolean cenario5, boolean cenario6) {

        boolean haveSelic = false;  // indica se há investimentos atrelados a Selic cadastrados
        boolean haveCDI = false;    // indica se há investimentos atrelados ao CDI cadastrados
        boolean haveTR = false;     // indica se há investimentos atrelados a TR cadastrados
        boolean haveIPCA = false;   // indica se há investimentos atrelados ao IPCA cadastrados

        // percorre todos os investimentos recebidos como parâmetros identificando se eles são
        // indexados, e identificando quais os índices utilizados, além de verificar se os
        // investimentos pré fixados estão adequadamente definidos.
        for (int i = 0; i < investimentos.size(); i++) {
            
            boolean isIndexado = false;

            // Regra para investimentos indexados ao CDI:
            // - LCI, LCA, CDB e RDB pós fixados são indexados ao CDI.
            if ((investimentos.get(i).getTipo() == Investimento.TIPO.LCI
                    || investimentos.get(i).getTipo() == Investimento.TIPO.LCA
                    || investimentos.get(i).getTipo() == Investimento.TIPO.CDB
                    || investimentos.get(i).getTipo() == Investimento.TIPO.RDB)) {

                if (((InvestimentosComunsSemelhantes) investimentos.get(i)).getTipoDeRentabilidade()
                        == InvestimentosComunsSemelhantes.TIPO_DE_RENTABILIDADE.POS_FIXADO) {
                    // o investimento em questão é indexado ao CDI
                    haveCDI = true;
                    isIndexado = true;
                }
            }
            
            // Regra para investimentos indexados ao IPCA
            // - Caso o investimento seja um Tesouro IPCA ele será indexado ao IPCA
            if (investimentos.get(i).getTipo() == Investimento.TIPO.TESOURO_IPCA) {
                haveIPCA = true;
                isIndexado = true;
            }

            // Regra para investimentos indexados ao SELIC
            // - Tesouro Selic e Poupança são Indexados ao SELIC.
            if (investimentos.get(i).getTipo() == Investimento.TIPO.TESOURO_SELIC) {
                haveSelic = true;
                isIndexado = true;
            }

            // Regra para investimentos indexados ao SELIC
            // - Tesouro Selic e Poupança são Indexados ao SELIC.
            // Regra para investimentos indexados a TR
            // - A poupança é indexada a TR
            if (investimentos.get(i).getTipo() == Investimento.TIPO.POUPANCA) {
                haveSelic = true;
                haveTR = true;
                isIndexado = true;
            }

            // caso o investimento não seja indexado
            if (!isIndexado) {
                // caso algum dos cenários solicitados não estejam preenchidos
                if (verificaSeCenarioEstaAdequadamenteDefinido(investimentos.get(i), cenario1, cenario2, cenario3, cenario4, cenario5, cenario6) == false)
                    return false;
            }
        }
        
        // verificando se os índices estão corretamente definidos
        GerenciaArmazenamentoIndices gerenciaArmazenamentoIndices = new GerenciaArmazenamentoIndices();
        Indice indice;

        if (haveCDI) {
            indice = gerenciaArmazenamentoIndices.pesquisaPorTipo(Indice.TIPO.CDI);

            if (indice == null) { //índice não cadastrado, tentando cadastra-lo
                gerenciaArmazenamentoIndices.salvaOuAtualiza(new Indice(Indice.TIPO.CDI));
                indice = gerenciaArmazenamentoIndices.pesquisaPorTipo(Indice.TIPO.CDI);
            }
            
            if (indice == null) {
                return false; // índice não cadastrado/erro ao cadastra-lo
            } else if (verificaSeCenarioEstaAdequadamenteDefinido(indice, cenario1, cenario2, cenario3, cenario4, cenario5, cenario6) == false) {
                return false; // índice não está com todos os cenários preenchidos
            }
        }

        if (haveSelic) {
            indice = gerenciaArmazenamentoIndices.pesquisaPorTipo(Indice.TIPO.SELIC);

            if (indice == null) { //índice não cadastrado, tentando cadastra-lo
                gerenciaArmazenamentoIndices.salvaOuAtualiza(new Indice(Indice.TIPO.SELIC));
                indice = gerenciaArmazenamentoIndices.pesquisaPorTipo(Indice.TIPO.SELIC);
            }
            
            if (indice == null) {
                return false; // índice não cadastrado/erro ao cadastra-lo
            } else if (verificaSeCenarioEstaAdequadamenteDefinido(indice, cenario1, cenario2, cenario3, cenario4, cenario5, cenario6) == false) {
                return false; // índice não está com todos os cenários preenchidos
            }
        }

        if (haveIPCA) {
            indice = gerenciaArmazenamentoIndices.pesquisaPorTipo(Indice.TIPO.IPCA);

            if (indice == null) { //índice não cadastrado, tentando cadastra-lo
                gerenciaArmazenamentoIndices.salvaOuAtualiza(new Indice(Indice.TIPO.IPCA));
                indice = gerenciaArmazenamentoIndices.pesquisaPorTipo(Indice.TIPO.IPCA);
            }
            
            if (indice == null) {
                return false; // índice não cadastrado/erro ao cadastra-lo
            } else if (verificaSeCenarioEstaAdequadamenteDefinido(indice, cenario1, cenario2, cenario3, cenario4, cenario5, cenario6) == false) {
                return false; // índice não está com todos os cenários preenchidos
            }
        }

        if (haveTR) {
            indice = gerenciaArmazenamentoIndices.pesquisaPorTipo(Indice.TIPO.TR);

            if (indice == null) { //índice não cadastrado, tentando cadastra-lo
                gerenciaArmazenamentoIndices.salvaOuAtualiza(new Indice(Indice.TIPO.TR));
                indice = gerenciaArmazenamentoIndices.pesquisaPorTipo(Indice.TIPO.TR);
            }
            
            if (indice == null) {
                return false; // índice não cadastrado/erro ao cadastra-lo
            } else if (verificaSeCenarioEstaAdequadamenteDefinido(indice, cenario1, cenario2, cenario3, cenario4, cenario5, cenario6) == false) {
                return false; // índice não está com todos os cenários preenchidos
            }
        }

        return true;
    }

    /**
     * Verifica se algum dos campos de cenário do Investimento recebido como parâmetro é nulo (inválido).
     * Obs. os cenários são verificados de acordo com o status do cenário recebidos como parâmetros.
     * 
     * @param entidade
     * @param cenario1
     * @param cenario2
     * @param cenario3
     * @param cenario4
     * @param cenario5
     * @param cenario6
     * @return 
     */
    private boolean verificaSeCenarioEstaAdequadamenteDefinido(Investimento entidade, boolean cenario1, boolean cenario2, boolean cenario3, boolean cenario4, boolean cenario5, boolean cenario6) {
        if (cenario1 == true && entidade.getValorCenario1() == null) {
            return false;
        }

        if (cenario2 == true && entidade.getValorCenario2() == null) {
            return false;
        }

        if (cenario3 == true && entidade.getValorCenario3() == null) {
            return false;
        }

        if (cenario4 == true && entidade.getValorCenario4() == null) {
            return false;
        }

        if (cenario5 == true && entidade.getValorCenario5() == null) {
            return false;
        }

        if (cenario6 == true && entidade.getValorCenario6() == null) {
            return false;
        }

        return true;
    }

    
    /**
     * Verifica se algum dos campos de cenário do Indice recebido como parâmetro é nulo (inválido).
     * Obs. os cenários são verificados de acordo com o status do cenário recebidos como parâmetros.
     * 
     * @param entidade
     * @param cenario1
     * @param cenario2
     * @param cenario3
     * @param cenario4
     * @param cenario5
     * @param cenario6
     * @return 
     */
    private boolean verificaSeCenarioEstaAdequadamenteDefinido(Indice entidade, boolean cenario1, boolean cenario2, boolean cenario3, boolean cenario4, boolean cenario5, boolean cenario6) {
        if (cenario1 && entidade.getValorCenario1() == null) {
            return false;
        }

        if (cenario2 && entidade.getValorCenario2() == null) {
            return false;
        }

        if (cenario3 && entidade.getValorCenario3() == null) {
            return false;
        }

        if (cenario4 && entidade.getValorCenario4() == null) {
            return false;
        }

        if (cenario5 && entidade.getValorCenario5() == null) {
            return false;
        }

        if (cenario6 && entidade.getValorCenario6() == null) {
            return false;
        }

        return true;
    }
}
