/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package br.com.flaviobarros.simuladordeinvestimentos.cenarios.model;

import br.com.flaviobarros.simuladordeinvestimentos.investimentos.model.Investimento;
import br.com.flaviobarros.simuladordeinvestimentos.investimentos.model.InvestimentosComunsSemelhantes;
import br.com.flaviobarros.simuladordeinvestimentos.investimentos.indices.controller.GerenciaArmazenamentoIndices;
import br.com.flaviobarros.simuladordeinvestimentos.investimentos.indices.model.Indice;
import java.util.ArrayList;
import java.util.List;
import javax.swing.table.AbstractTableModel;

/**
 *
 * @author flavio
 * 
 * Padronizações adotadas:
 * - a janela exibe:
 *      - primeiro: os indícadores econômicos vinculados a investimentos cadastrado;
 *      - segundo: investimentos não indexados.
 * 
 */
public class AbstractTableModelCenarios extends AbstractTableModel {

//    contem os investimentos não indexados a nenhum indicador financeiro
    private List<Investimento> investimentosNaoIndexados;
    
//    contem os investimentos indexados a a algum indicador financeiro
    private List<Investimento> investimentosIndexados;
    
//    contém uma lista com os indices necessários de acordo com os investimentos cadastrados
//    e exibidos na janela de alteração de cenários (JTable da CenariosView)
    private List<Indice> indices;
    
//    indica se a tabela encontra-se em modo de edição de cenários ou não.
    boolean statusEdicao;
    
//    indica se há investimentos atrelados a Selic cadastrados
    boolean haveSelic;
    
//    indica se há investimentos atrelados ao CDI cadastrados
    boolean haveCDI;
    
//    indica se há investimentos atrelados a TR cadastrados
    boolean haveTR;
    
//    indica se há investimentos atrelados ao IPCA cadastrados
    boolean haveIPCA;
    
//    contém o título das colunas exibidos na tabela
    private final String[] colunas;

    
    public AbstractTableModelCenarios() {
        iniciaEstruturas();
        
        this.colunas = new String[]{"Investimento", "Cenário 1", "Cenário 2", "Cenário 3", "Cenário 4", "Cenário 5", "Cenário 6"};
        this.statusEdicao = false;
        
    }
    
    private void iniciaEstruturas() {
        this.investimentosNaoIndexados = new ArrayList<>();
        this.investimentosIndexados = new ArrayList<>();
        this.indices = new ArrayList<>();
        
        this.haveCDI = false;
        this.haveSelic = false;
        this.haveTR = false;
        this.haveIPCA = false;
    }

    @Override
    public String getColumnName(int column) {
        return this.colunas[column];
    }

    @Override
    public int getRowCount() {
//        a tabela exibe os investimentos não indexados a indicadores 
//        "+"
//        os indicadores atrelados a investimentos cadastrados.
//        daí a forma de calculo da quantidade de linhas.
        return investimentosNaoIndexados.size()+indices.size();
    }

    @Override
    public int getColumnCount() {
        return colunas.length;
    }

    @Override
    public Object getValueAt(int rowIndex, int columnIndex) {

        // caso o item seja um ídice
        if (rowIndex < indices.size()){
            
            switch (columnIndex) {
                case 0:
                    return this.getDetalhamentoCenario(rowIndex);
                case 1:
                    return indices.get(rowIndex).getValorCenario1();
                case 2:
                    return indices.get(rowIndex).getValorCenario2();
                case 3:
                    return indices.get(rowIndex).getValorCenario3();
                case 4:
                    return indices.get(rowIndex).getValorCenario4();
            case 5:
                    return indices.get(rowIndex).getValorCenario5();
                case 6:
                    return indices.get(rowIndex).getValorCenario6();
                default:
                    return null;
            }  
            
        } else { // caso o item seja um investimento não indexado
            
            switch (columnIndex) {
                case 0:
                    return this.getDetalhamentoCenario(rowIndex);
                case 1:
                    return investimentosNaoIndexados.get(rowIndex-indices.size()).getValorCenario1();
                case 2:
                    return investimentosNaoIndexados.get(rowIndex-indices.size()).getValorCenario2();
                case 3:
                    return investimentosNaoIndexados.get(rowIndex-indices.size()).getValorCenario3();
                case 4:
                    return investimentosNaoIndexados.get(rowIndex-indices.size()).getValorCenario4();
            case 5:
                    return investimentosNaoIndexados.get(rowIndex-indices.size()).getValorCenario5();
                case 6:
                    return investimentosNaoIndexados.get(rowIndex-indices.size()).getValorCenario6();
                default:
                    return null;
            }            
            
        }

    }

    @Override
    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {

        Double valor = validateDouble(aValue);
        
        if (valor == null) {
            valor = 0.0;
        }

        // caso o item editado tenha sido um ídice
        if (rowIndex < indices.size()){
            switch (columnIndex) {
                case 1:
                    indices.get(rowIndex).setValorCenario1(valor);
                    break;
                case 2:
                    indices.get(rowIndex).setValorCenario2(valor);
                    break;
                case 3:
                    indices.get(rowIndex).setValorCenario3(valor);
                    break;
                case 4:
                    indices.get(rowIndex).setValorCenario4(valor);
                    break;
                case 5:
                    indices.get(rowIndex).setValorCenario5(valor);
                    break;
                case 6:
                    indices.get(rowIndex).setValorCenario6(valor);
                    break;
                default:
                    break;
            }
        } else { // caso o elemento editado tenha sido um investimento não indexado
            
            switch (columnIndex) {
                case 1:
                    investimentosNaoIndexados.get(rowIndex-indices.size()).setValorCenario1(valor);
                    break;
                case 2:
                    investimentosNaoIndexados.get(rowIndex-indices.size()).setValorCenario2(valor);
                    break;
                case 3:
                    investimentosNaoIndexados.get(rowIndex-indices.size()).setValorCenario3(valor);
                    break;
                case 4:
                    investimentosNaoIndexados.get(rowIndex-indices.size()).setValorCenario4(valor);
                    break;
                case 5:
                    investimentosNaoIndexados.get(rowIndex-indices.size()).setValorCenario5(valor);
                    break;
                case 6:
                    investimentosNaoIndexados.get(rowIndex-indices.size()).setValorCenario6(valor);
                    break;
                default:
                    break;
            }
        }

        fireTableDataChanged();
    }

    @Override
    public Class<?> getColumnClass(int column) {
        
//    *** Eu poderia configurar a célula para só aceitar tipos Double por aqui,
//        mas achei mais amigavel para o usuário validar o tipo do dado inserido
//        manualmente.
//        return column >= 1 ? Double.class : super.getColumnClass(column);

        return super.getColumnClass(column);
    }

//    public Investimento getInvestimento(int rowIndex) {
//        return investimentos.get(rowIndex);
//    }

//    public void addRow(Investimento c) {
//        this.investimentos.add(c);
//        fireTableDataChanged();
//    }

    public void setDados(List<Investimento> investimentos) {

        if (investimentos != null) {
            separaInvestimentosIndexadosNaoIndexados(investimentos);
        }

        fireTableDataChanged();
    }

    @Override
    public boolean isCellEditable(int rowIndex, int columnIndex) {

        if (columnIndex == 0) { // definindo primeira coluna como não editável
            return false;
        } else {
            return getStatusEdicao();//se a tabela estiver com o módulo de edição ativa retorna true,
                                     // do contrário retorna false.
        }

    }

    
    public void habilitaEdicao() {
        this.statusEdicao = true;
    }

    public void desabilitaEdicao() {
        this.statusEdicao = false;
    }

    public List<Investimento> getDadosInvestimentos() {
        return uneInvestimentosIndexadosNaoIndexados();
    }

    public List<Indice> getDadosIndices() {
        return indices;
    }
    
//    getStatusEdicao();
//    - retorna um boolean indicando se a tabela está ou não em modo de edição de cenários
    private boolean getStatusEdicao() {
        return statusEdicao;
    }
    
    private Object getDetalhamentoCenario(int rowIndex) {
        
//        contando a quantidade de índices econômicos ativos e atualmente exibidos;
        int conter = 0;
        if (haveSelic) conter++;
        if (haveCDI) conter++;
        if (haveTR) conter++;
        if (haveIPCA) conter++;
        
        if(rowIndex < conter){ // caso o detalhamento a ser exibido seja de um indicador econômico.
            return indices.get(rowIndex).getNome();
        } else { // caso o detalhamento a ser exibido seja de um investimer econômico. 
           return investimentosNaoIndexados.get(rowIndex-conter).getNome();
        }
    }
    
    
    
    
    
    
//------------------------------------------------------------------------------------------------------//  
//--- FUNCÕES PRIVADAS AUXILIARES ----------------------------------------------------------------------//
//------------------------------------------------------------------------------------------------------//    
    
//    validateDouble(Object);
//    - caso o objeto recebido como parâmetro seja uma String contendo que contenha a representação válida
//    de um Double o Double em questão sera retornado (retorna Double);
//    - caso o objeto recebido como parâmetro não seja uma string contendo um Double válido retorna NULL.
    private Double validateDouble(Object numberObj) {
        
        try {
            
            String strValor = (String) numberObj;
            
            strValor = strValor.replace(',', '.');
            
            Double number = Double.parseDouble(strValor);
            
            return number;
            
        } catch (NumberFormatException e) {
            return null;
        }
        
    }
    
//    separaInvestimentosIndexadosNaoIndexados(List<Investimento>);
//    - recebe uma lista de investimentos populando os arrays:
//      -- indices: com os indices utilizados pelos investimentos recebidos;
//      -- investimentosIndexados: com investimentos indexados a algum indicador econômico
//      -- investimentosNaoIndexados: com investimentos indexados a algum indicador econômico
//    esta separação é importante para a exibição correta dos indíces e investimentos para definição dos cenários:
//      - investimentos indexados tem seus cenários definidos com base em seus índices
//      - investimentos não indexados tem seus cenários configurados pelos dados diretamente inseridos pelos usuários.
    private void separaInvestimentosIndexadosNaoIndexados(List<Investimento> investimentos) {
        
        iniciaEstruturas();
        
        for (int i = 0; i < investimentos.size(); i++) {
            
            boolean isIndexado = false;
            
            // Regra para investimentos indexados ao CDI:
            // - LCI, LCA, CDB e RDB pós fixados são indexados ao CDI.
            if((investimentos.get(i).getTipo() == Investimento.TIPO.LCI ||
                investimentos.get(i).getTipo() == Investimento.TIPO.LCA ||
                investimentos.get(i).getTipo() == Investimento.TIPO.CDB ||
                investimentos.get(i).getTipo() == Investimento.TIPO.RDB)) {
                
                if (((InvestimentosComunsSemelhantes)investimentos.get(i)).getTipoDeRentabilidade() ==
                        InvestimentosComunsSemelhantes.TIPO_DE_RENTABILIDADE.POS_FIXADO){
                   
                    // o investimento em questão é indexado ao CDI
                    haveCDI = true;
                    isIndexado = true;
                    
                }  
            }
            
            // Regra para investimentos indexados ao IPCA
            // - Caso o investimento seja um Tesouro IPCA ele será indexado ao IPCA
            if (investimentos.get(i).getTipo() == Investimento.TIPO.TESOURO_IPCA){
                    haveIPCA = true;
                    isIndexado = true;
            }
            
            // Regra para investimentos indexados ao SELIC
            // - Tesouro Selic e Poupança são Indexados ao SELIC.
            if (investimentos.get(i).getTipo() == Investimento.TIPO.TESOURO_SELIC){
                    haveSelic = true;
                    isIndexado = true;
            }
            
            // Regra para investimentos indexados ao SELIC
            // - Tesouro Selic e Poupança são Indexados ao SELIC.
            // Regra para investimentos indexados a TR
            // - A poupança é indexada a TR
            if (investimentos.get(i).getTipo() == Investimento.TIPO.POUPANCA){
                    haveSelic = true;
                    haveTR = true;
                    isIndexado = true;
            }
            
            // separando efetivamente as listas de acordo com seus tipos
            if (isIndexado) {
                investimentosIndexados.add(investimentos.get(i));
            } else {
                investimentosNaoIndexados.add(investimentos.get(i));
            }
        }
        
        // popula o array de índices de acordo com a necessidade
        populaArrayIndices();

    }
    
//  populaArrayIndices():
//      - popula o array de índices de acordo com a necessidade.  
    private void populaArrayIndices() {
        
        GerenciaArmazenamentoIndices gerenciaArmazenamentoIndices = new GerenciaArmazenamentoIndices();
        Indice indice;
        
        if(haveCDI){
            indice = gerenciaArmazenamentoIndices.pesquisaPorTipo(Indice.TIPO.CDI);
            
            if(indice == null){
                indice = new Indice(Indice.TIPO.CDI);
            }
                
            indices.add(indice);
        }
        
        if(haveSelic){
            indice = gerenciaArmazenamentoIndices.pesquisaPorTipo(Indice.TIPO.SELIC);
            
            if(indice == null){
                indice = new Indice(Indice.TIPO.SELIC);
            }
                
            indices.add(indice);
        }
        
        if(haveIPCA){
            indice = gerenciaArmazenamentoIndices.pesquisaPorTipo(Indice.TIPO.IPCA);
            
            if(indice == null){
                indice = new Indice(Indice.TIPO.IPCA);
            }
             
            indices.add(indice);
        }
        
        if(haveTR){
            indice = gerenciaArmazenamentoIndices.pesquisaPorTipo(Indice.TIPO.TR);
            
            if(indice == null){
                indice = new Indice(Indice.TIPO.TR);
            }
                
            indices.add(indice);
        }
        
    }
    
//    uneInvestimentosIndexadosNaoIndexados();
//    - retorna um array com os investimentos indexados e não indexados,
//      sendo que os indexados já estão com seus valores configurados de 
//      acordo com os  valores dos índices informados pelo usuário para
//      cada cenário.
    private List<Investimento> uneInvestimentosIndexadosNaoIndexados(){
               
        List<Investimento> investimentos = new ArrayList<>();
        
        Indice indiceCDI = null;
        Indice indiceSelic = null;
        Indice indiceIPCA = null;
        Indice indiceTR = null;//---------------------------------------------------problemas com a remuneração da poupança e a edição de cenários
                               //---------------------------------------------------os índices também deverão ser persistidos
                               //---------------------------------------------------
        
        for (int i = 0; i < this.indices.size(); i++) {
            
            if (this.indices.get(i).getTipo() == Indice.TIPO.CDI)
                indiceCDI = this.indices.get(i);
                
            if (this.indices.get(i).getTipo() == Indice.TIPO.SELIC)
                indiceSelic = this.indices.get(i);  
            
            if (this.indices.get(i).getTipo() == Indice.TIPO.IPCA)
                indiceIPCA = this.indices.get(i);
            
            if (this.indices.get(i).getTipo() == Indice.TIPO.TR)
                indiceTR = this.indices.get(i);
        }
        
        investimentos.addAll(investimentosNaoIndexados);
        
        for (int i = 0; i < investimentosIndexados.size(); i++) {
            
//            // Regra para investimentos indexados ao CDI:
//            // - LCI, LCA, CDB e RDB pós fixados são indexados ao CDI.
//            if((investimentosIndexados.get(i).getTipo() == Investimento.TIPO.LCI ||
//                investimentosIndexados.get(i).getTipo() == Investimento.TIPO.LCA ||
//                investimentosIndexados.get(i).getTipo() == Investimento.TIPO.CDB ||
//                investimentosIndexados.get(i).getTipo() == Investimento.TIPO.RDB)) {
//                
//                if (((InvestimentosComunsSemelhantes)investimentosIndexados.get(i)).getTipoDeRentabilidade() ==
//                        InvestimentosComunsSemelhantes.TIPO_DE_RENTABILIDADE.POS_FIXADO){
//                   
//                    // o investimento em questão é indexado ao CDI
//                    copiaCenario(indiceCDI, investimentosIndexados.get(i));
//                }  
//            }
//            
//            // Regra para investimentos indexados ao IPCA
//            // - Caso o investimento seja um Tesouro IPCA ele será indexado ao IPCA
//            if (investimentosIndexados.get(i).getTipo() == Investimento.TIPO.TESOURO_IPCA){
//                copiaCenario(indiceIPCA, investimentosIndexados.get(i));
//            }
//            
//            // Regra para investimentos indexados ao SELIC
//            // - Tesouro Selic e Poupança são Indexados ao SELIC.
//            if (investimentosIndexados.get(i).isTesouroSelic() || investimentosIndexados.get(i).isPoupanca()){
//                    copiaCenario(indiceSelic, investimentosIndexados.get(i));
//            }
//            
//            // Regra para investimentos indexados a TR
//            // - A poupança é indexada a TR
//            if (investimentosIndexados.get(i).getTipo() == Investimento.TIPO.POUPANCA){
////                 copiaCenarioPoupanca(indiceSelic, indiceTR, (InvestimentoPoupanca)investimentosIndexados.get(i));
//                 copiaCenario(indiceTR, investimentosIndexados.get(i));
//            }
            
            // adiciona efetivamente o invetimento indexado a lista de investimento geral
            investimentos.add(investimentosIndexados.get(i));
        }
        
        return investimentos;
        
    }
    
//    copiaCenario(Indice origem, Investimento destino);
//    - copia os dados do cenário do indice origem nos dados do cenário do índice destino.
    private void copiaCenario(Indice origem, Investimento destino) {
        destino.setValorCenario1(origem.getValorCenario1());
        destino.setValorCenario2(origem.getValorCenario2());
        destino.setValorCenario3(origem.getValorCenario3());
        destino.setValorCenario4(origem.getValorCenario4());
        destino.setValorCenario5(origem.getValorCenario5());
        destino.setValorCenario6(origem.getValorCenario6());
    }
    

}
