/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package br.com.flaviobarros.simuladordeinvestimentos.investimentos.controller;

import br.com.flaviobarros.simuladordeinvestimentos.investimentos.indices.model.Indice;
import br.com.flaviobarros.simuladordeinvestimentos.investimentos.model.Investimento;
import br.com.flaviobarros.simuladordeinvestimentos.investimentos.model.InvestimentoCDB;
import br.com.flaviobarros.simuladordeinvestimentos.investimentos.model.InvestimentoFundos;
import br.com.flaviobarros.simuladordeinvestimentos.investimentos.model.InvestimentoLCA;
import br.com.flaviobarros.simuladordeinvestimentos.investimentos.model.InvestimentoLCI;
import br.com.flaviobarros.simuladordeinvestimentos.investimentos.model.InvestimentoPoupanca;
import br.com.flaviobarros.simuladordeinvestimentos.investimentos.model.InvestimentoPrevidencia;
import br.com.flaviobarros.simuladordeinvestimentos.investimentos.model.InvestimentoRDB;
import br.com.flaviobarros.simuladordeinvestimentos.investimentos.model.InvestimentoTesouroIPCA;
import br.com.flaviobarros.simuladordeinvestimentos.investimentos.model.InvestimentoTesouroSelic;
import br.com.flaviobarros.simuladordeinvestimentos.relatorio.model.RentabilidadeInvestimento;
import java.util.Calendar;
import java.util.GregorianCalendar;

/**
 *
 * @author flavio
 */
public class CalculaRentabilidade {

    private final boolean CONSIDERAR_PRAZO_VENCIMENTO_APLICACOES = true;
    
    private final Indice indiceCDI;
    private final Indice indiceSelic;
    private final Indice indiceIPCA;
    private final Indice indiceTR;
    
    private enum CENARIO {
        A,
        B,
        C,
        D,
        E,
        F
    }

    /* No útimo dia útil dos meses de maio e novembro ocorre a cobrança de Imposto de 
     * Renda conforme a menor aliquota para o tipo de fundo (fundos de curto prazo 20% fundos
     * de longo prazo 15%), esta cobrança é realizada reduzindo o número de cotas detidas pelo
     * investidor, daı́ o nome come cotas (Resende, 2017)_
     * No momento do resgate do investimento o investidor realizará o pagamento da dife-
     * rença entre a aliquota de imposto de renda devida de acordo com o tempo que o investidor
     * permaneceu no plano e o valor de imposto de renda já pago atravéz do come cotas_ Nos
     * fundos de investimentos em ações não há come cotas (Resende, 2017).
    */
    public final Double PERCENTUAL_IR_COME_COTAS_FUNDOS_CURTO_PRAZO = 20.0;
    public final Double PERCENTUAL_IR_COME_COTAS_FUNDOS_LONGO_PRAZO = 15.0;
    

    public CalculaRentabilidade(Indice indiceCDI, Indice indiceSelic, Indice indiceIPCA, Indice indiceTR) {
        this.indiceCDI = indiceCDI;
        this.indiceSelic = indiceSelic;
        this.indiceIPCA = indiceIPCA;
        this.indiceTR = indiceTR;
    }

    public RentabilidadeInvestimento calcula(Investimento investimento, int prazo, int intervaloEmMesCalculo, Double valorInvestido, Calendar mesAnoAplicacao) {
        if (investimento.isLCI()) {
            return calculaRentabilidadeLCI((InvestimentoLCI) investimento, prazo, intervaloEmMesCalculo, valorInvestido, mesAnoAplicacao);
        } else if (investimento.isLCA()) {
            return calculaRentabilidadeLCA((InvestimentoLCA) investimento, prazo, intervaloEmMesCalculo, valorInvestido, mesAnoAplicacao);
        } else if (investimento.isCDB()) {
            return calculaRentabilidadeCDB((InvestimentoCDB) investimento, prazo, intervaloEmMesCalculo, valorInvestido, mesAnoAplicacao);
        } else if (investimento.isRDB()) {
            return calculaRentabilidadeRDB((InvestimentoRDB) investimento, prazo, intervaloEmMesCalculo, valorInvestido, mesAnoAplicacao);
        } else if (investimento.isPoupanca()) {
            return calculaRentabilidadePoupanca((InvestimentoPoupanca) investimento, prazo, intervaloEmMesCalculo, valorInvestido);
        } else if (investimento.isTesouroSelic()) {
            return calculaRentabilidadeTesouroSelic((InvestimentoTesouroSelic) investimento, prazo, intervaloEmMesCalculo, valorInvestido, mesAnoAplicacao);
        } else if (investimento.isTesouroIPCA()) {
            return calculaRentabilidadeTesouroIPCA((InvestimentoTesouroIPCA) investimento, prazo, intervaloEmMesCalculo, valorInvestido, mesAnoAplicacao);
        } else if (investimento.isPrevidencia()) {
            return calculaRentabilidadePrevidencia((InvestimentoPrevidencia) investimento, prazo, intervaloEmMesCalculo, valorInvestido);
        } else if (investimento.isFundoDeInvestimento()) {
            return calculaRentabilidadeFundos((InvestimentoFundos)investimento, prazo, intervaloEmMesCalculo, valorInvestido, mesAnoAplicacao);
        }

        throw new IllegalArgumentException("Tipo do Investimento recebido como parâmetro inválido!");
    }

    // Aproximei meses para 30 dias
    private Double deduzTriburacaoIRRendaFixaBase(Double rendimento, int qtdMeses) {

        int dias = qtdMeses * 30;

        Double tributacao;

        if (dias > 720) {
            tributacao = 15.00;
        } else if (dias > 360) {
            tributacao = 17.50;
        } else if (dias > 180) {
            tributacao = 20.00;
        } else {
            tributacao = 22.50;
        }

        return rendimento * (1 - (tributacao / 100));
    }
    
    

    
    
    
//------------------------------------------ CALCULOS REFERENTES A FUNDOS -----------------------------------------------------------------
    private RentabilidadeInvestimento calculaRentabilidadeFundos(InvestimentoFundos investimentoFundos, final int prazo, final int intervaloEmMesCalculo, final Double valorInvestido, Calendar mesAnoAplicacao) {
        
        final int MAIO = 5, NOVEMBRO = 11;
        
        RentabilidadeInvestimento resultado = new RentabilidadeInvestimento(investimentoFundos);
        
        Double rendimentosAcumuladosComComeCotasA = 0.0;
        Double rendimentosAcumuladosComComeCotasB = 0.0;
        Double rendimentosAcumuladosComComeCotasC = 0.0;
        Double rendimentosAcumuladosComComeCotasD = 0.0;
        Double rendimentosAcumuladosComComeCotasE = 0.0;
        Double rendimentosAcumuladosComComeCotasF = 0.0;
        
        Double acumuladoSemComeCotasCenarioA = 0.0;
        Double acumuladoSemComeCotasCenarioB = 0.0;
        Double acumuladoSemComeCotasCenarioC = 0.0;
        Double acumuladoSemComeCotasCenarioD = 0.0;
        Double acumuladoSemComeCotasCenarioE = 0.0;
        Double acumuladoSemComeCotasCenarioF = 0.0;
        
        // passando a taxa de rentabilidade de anual para mensal
        Double taxaAoMesCenarioA = convertePercentualAnualParaMensal(investimentoFundos.getValorCenario1());
        Double taxaAoMesCenarioB = convertePercentualAnualParaMensal(investimentoFundos.getValorCenario2());
        Double taxaAoMesCenarioC = convertePercentualAnualParaMensal(investimentoFundos.getValorCenario3());
        Double taxaAoMesCenarioD = convertePercentualAnualParaMensal(investimentoFundos.getValorCenario4());
        Double taxaAoMesCenarioE = convertePercentualAnualParaMensal(investimentoFundos.getValorCenario5());
        Double taxaAoMesCenarioF = convertePercentualAnualParaMensal(investimentoFundos.getValorCenario6());
        
        // convertendo a taxa de formato percentual para decimal ( ex: 1% = 0,01)
        Double taxaCenarioA = taxaAoMesCenarioA / 100;
        Double taxaCenarioB = taxaAoMesCenarioB / 100;
        Double taxaCenarioC = taxaAoMesCenarioC / 100;
        Double taxaCenarioD = taxaAoMesCenarioD / 100;
        Double taxaCenarioE = taxaAoMesCenarioE / 100;
        Double taxaCenarioF = taxaAoMesCenarioF / 100;
        
        Integer mesAtual = mesAnoAplicacao.get(GregorianCalendar.MONTH) + 1; // mês de 1 a 12;
        
        for (int i = 1; i <= prazo; i++) {
            
            acumuladoSemComeCotasCenarioA += (rendimentosAcumuladosComComeCotasA + acumuladoSemComeCotasCenarioA + valorInvestido) * taxaCenarioA;
            acumuladoSemComeCotasCenarioB += (rendimentosAcumuladosComComeCotasB + acumuladoSemComeCotasCenarioB + valorInvestido) * taxaCenarioB;
            acumuladoSemComeCotasCenarioC += (rendimentosAcumuladosComComeCotasC + acumuladoSemComeCotasCenarioC + valorInvestido) * taxaCenarioC;
            acumuladoSemComeCotasCenarioD += (rendimentosAcumuladosComComeCotasD + acumuladoSemComeCotasCenarioD + valorInvestido) * taxaCenarioD;
            acumuladoSemComeCotasCenarioE += (rendimentosAcumuladosComComeCotasE + acumuladoSemComeCotasCenarioE + valorInvestido) * taxaCenarioE;
            acumuladoSemComeCotasCenarioF += (rendimentosAcumuladosComComeCotasF + acumuladoSemComeCotasCenarioF + valorInvestido) * taxaCenarioF;
            
            // incidência do come cotas
            if (mesAtual == MAIO || mesAtual == NOVEMBRO) {
                
                rendimentosAcumuladosComComeCotasA += descontaComeCotasFundos(investimentoFundos, acumuladoSemComeCotasCenarioA);
                rendimentosAcumuladosComComeCotasB += descontaComeCotasFundos(investimentoFundos, acumuladoSemComeCotasCenarioB);
                rendimentosAcumuladosComComeCotasC += descontaComeCotasFundos(investimentoFundos, acumuladoSemComeCotasCenarioC);
                rendimentosAcumuladosComComeCotasD += descontaComeCotasFundos(investimentoFundos, acumuladoSemComeCotasCenarioD);
                rendimentosAcumuladosComComeCotasE += descontaComeCotasFundos(investimentoFundos, acumuladoSemComeCotasCenarioE);
                rendimentosAcumuladosComComeCotasF += descontaComeCotasFundos(investimentoFundos, acumuladoSemComeCotasCenarioF);
                
                acumuladoSemComeCotasCenarioA = 0.0;
                acumuladoSemComeCotasCenarioB = 0.0;
                acumuladoSemComeCotasCenarioC = 0.0;
                acumuladoSemComeCotasCenarioD = 0.0;
                acumuladoSemComeCotasCenarioE = 0.0;
                acumuladoSemComeCotasCenarioF = 0.0;
                
            }
            
            if(i % intervaloEmMesCalculo == 0) {
                
                int dias = i * 30;
                                
                Double montanteA = valorInvestido + deduzTributacaoIRFundos(investimentoFundos, rendimentosAcumuladosComComeCotasA, acumuladoSemComeCotasCenarioA, dias);
                Double montanteB = valorInvestido + deduzTributacaoIRFundos(investimentoFundos, rendimentosAcumuladosComComeCotasB, acumuladoSemComeCotasCenarioB, dias);
                Double montanteC = valorInvestido + deduzTributacaoIRFundos(investimentoFundos, rendimentosAcumuladosComComeCotasC, acumuladoSemComeCotasCenarioC, dias);
                Double montanteD = valorInvestido + deduzTributacaoIRFundos(investimentoFundos, rendimentosAcumuladosComComeCotasD, acumuladoSemComeCotasCenarioD, dias);
                Double montanteE = valorInvestido + deduzTributacaoIRFundos(investimentoFundos, rendimentosAcumuladosComComeCotasE, acumuladoSemComeCotasCenarioE, dias);
                Double montanteF = valorInvestido + deduzTributacaoIRFundos(investimentoFundos, rendimentosAcumuladosComComeCotasF, acumuladoSemComeCotasCenarioF, dias);

                resultado.addRentabilidadeMes(montanteA, montanteB, montanteC, montanteD, montanteE, montanteF);
            }
            
            mesAtual = (mesAtual+1) % 12;
            
        }
        
        return resultado;
    }
    
    private Double descontaComeCotasFundos(InvestimentoFundos fundo, Double valorRendimentosPosUltimoComeCotas) {
        
        if (valorRendimentosPosUltimoComeCotas > 0) /* caso o rendimento do fundo seja positivo deduz o come cotas */{
            
            if (fundo.isFundoDeCurtoPrazo())
                return valorRendimentosPosUltimoComeCotas * (1-(PERCENTUAL_IR_COME_COTAS_FUNDOS_CURTO_PRAZO/100));
            else if (fundo.isFundoDeLongoPrazo())
                return valorRendimentosPosUltimoComeCotas * (1-(PERCENTUAL_IR_COME_COTAS_FUNDOS_LONGO_PRAZO/100));
            else if (fundo.isFundoDeAcoes())
                return valorRendimentosPosUltimoComeCotas; //fundos de ações não possuem cobraça de come cotas
            
        } else {
            return valorRendimentosPosUltimoComeCotas;
        }
        
        throw new IllegalArgumentException("Tipo do fundo não informado para o calculo do come cotas!");
    }
    
    /**
     * Função deduzTributacaoIRFundos(InvestimentoFundos, Double, Double, int);
     * 
     * Fundos de Investimento em ações 15% de IR;
     * 
     * Fundos de Investimento de Curto Prazo;
     * - até 180 dias       : 22,5 % de IR;
     * - acima de 180 dias  : 20 % de IR;
     * 
     * Fundos de Investimento de Longo Prazo;
     * - até 180 dias       : 22,5 % de IR;
     * - de 181 a 360 dias  : 20 % de IR;
     * - de 361 a 720 dias  : 17,5 % de IR;
     * - acima de 720 dias  : 15 % de IR;
     * 
     * 
     * @param fundo
     * @param rentabilidadeComComeCotas
     * @param rentabilidadeSemComeCotas
     * @param diasDePermanenciaDoInvestimento
     * 
     * @return 
     */
    private Double deduzTributacaoIRFundos(InvestimentoFundos fundo, Double rentabilidadeComComeCotas, Double rentabilidadeSemComeCotas, int diasDePermanenciaDoInvestimento) {
        
        if (rentabilidadeComComeCotas >= 0 && rentabilidadeSemComeCotas >= 0) {
            
            if (fundo.isFundoDeCurtoPrazo()) {

                if (diasDePermanenciaDoInvestimento > 180) // 20 % de IR
                    return rentabilidadeComComeCotas + (rentabilidadeSemComeCotas * 0.80);
                else // 22,5 % de IR
                    return (rentabilidadeComComeCotas * 0.975) + (rentabilidadeSemComeCotas * 0.775);

            } else if (fundo.isFundoDeLongoPrazo()) {

                if(diasDePermanenciaDoInvestimento > 720) // 15 % de IR
                    return rentabilidadeComComeCotas + (rentabilidadeSemComeCotas * 0.85);
                else if(diasDePermanenciaDoInvestimento > 360) // 17,5 % de IR
                    return (rentabilidadeComComeCotas * 0.975) + (rentabilidadeSemComeCotas * 0.825);
                else if(diasDePermanenciaDoInvestimento > 180) // 20 % de IR
                    return (rentabilidadeComComeCotas * 0.95) + (rentabilidadeSemComeCotas * 0.8);
                else // 22,5 % de IR
                    return (rentabilidadeComComeCotas * 0.925) + (rentabilidadeSemComeCotas * 0.775);         

            } else if (fundo.isFundoDeAcoes()) { 

                // fundo de investimento em ações: 15 % de IR
                return (rentabilidadeComComeCotas + rentabilidadeSemComeCotas) * 0.85;

            }
        
        } else {
            // não deduzo IR caso a rentabilidade tenha sido negativa
            return rentabilidadeComComeCotas + rentabilidadeSemComeCotas;
        }
        
        throw new IllegalArgumentException("Tipo do fundo não informado para o calculo do come cotas!");
    }
    
    
    
    
//------------------------------------------ CALCULOS REFERENTES A PREVIDÊNCIA -----------------------------------------------------------------
      private RentabilidadeInvestimento calculaRentabilidadePrevidencia(InvestimentoPrevidencia investimentoPrevidencia, final int prazo, final int intervaloEmMesCalculo, final Double valorInvestido) {
        
        RentabilidadeInvestimento resultado = new RentabilidadeInvestimento(investimentoPrevidencia);
        
        Double rentabilidadeA;
        Double rentabilidadeB;
        Double rentabilidadeC;
        Double rentabilidadeD;
        Double rentabilidadeE;
        Double rentabilidadeF;
        
        // passando a taxa de rentabilidade de anual para mensal
        Double taxaAoMesCenarioA = convertePercentualAnualParaMensal(investimentoPrevidencia.getValorCenario1());
        Double taxaAoMesCenarioB = convertePercentualAnualParaMensal(investimentoPrevidencia.getValorCenario2());
        Double taxaAoMesCenarioC = convertePercentualAnualParaMensal(investimentoPrevidencia.getValorCenario3());
        Double taxaAoMesCenarioD = convertePercentualAnualParaMensal(investimentoPrevidencia.getValorCenario4());
        Double taxaAoMesCenarioE = convertePercentualAnualParaMensal(investimentoPrevidencia.getValorCenario5());
        Double taxaAoMesCenarioF = convertePercentualAnualParaMensal(investimentoPrevidencia.getValorCenario6());
        
        for (int i = intervaloEmMesCalculo; i <= prazo; i = i + intervaloEmMesCalculo) {
            rentabilidadeA = this.rentabilidadePrevidencia(investimentoPrevidencia, valorInvestido, taxaAoMesCenarioA, i, investimentoPrevidencia.getTipoPrevidencia(), investimentoPrevidencia.getTributacaoPrevidencia());
            rentabilidadeB = this.rentabilidadePrevidencia(investimentoPrevidencia, valorInvestido, taxaAoMesCenarioB, i, investimentoPrevidencia.getTipoPrevidencia(), investimentoPrevidencia.getTributacaoPrevidencia());
            rentabilidadeC = this.rentabilidadePrevidencia(investimentoPrevidencia, valorInvestido, taxaAoMesCenarioC, i, investimentoPrevidencia.getTipoPrevidencia(), investimentoPrevidencia.getTributacaoPrevidencia());
            rentabilidadeD = this.rentabilidadePrevidencia(investimentoPrevidencia, valorInvestido, taxaAoMesCenarioD, i, investimentoPrevidencia.getTipoPrevidencia(), investimentoPrevidencia.getTributacaoPrevidencia());
            rentabilidadeE = this.rentabilidadePrevidencia(investimentoPrevidencia, valorInvestido, taxaAoMesCenarioE, i, investimentoPrevidencia.getTipoPrevidencia(), investimentoPrevidencia.getTributacaoPrevidencia());
            rentabilidadeF = this.rentabilidadePrevidencia(investimentoPrevidencia, valorInvestido, taxaAoMesCenarioF, i, investimentoPrevidencia.getTipoPrevidencia(), investimentoPrevidencia.getTributacaoPrevidencia());
            
            resultado.addRentabilidadeMes(rentabilidadeA, rentabilidadeB, rentabilidadeC, rentabilidadeD, rentabilidadeE, rentabilidadeF);
        }
        
        return resultado;
    }
    
    private Double rentabilidadePrevidencia(InvestimentoPrevidencia investimento, Double valorInvestido, Double rentabililidadeAoMes, int mes, InvestimentoPrevidencia.TIPO_PREVIDENCIA tipo, InvestimentoPrevidencia.TRIBUTACAO_PREVIDENCIA tributacao) {
        
        Double resultado = null;
        
        if (null == tipo) { 
            
            throw new IllegalArgumentException("Previdência recebida como parâmetro não está como o TIPO (VGBL/PGBL) setado!");
            
        } else switch (tipo) {
            case VGBL:
                return rentabilidadePrevidenciaVGBL(valorInvestido, rentabililidadeAoMes, mes, tributacao, investimento.getTaxaDeCarregamentoAntecipada(), investimento.getTaxaDeCarregamentoPostecipada());
                
            default:
                return rentabilidadePrevidenciaPGBL(valorInvestido, rentabililidadeAoMes, mes, tributacao, investimento.getTaxaDeCarregamentoAntecipada(), investimento.getTaxaDeCarregamentoPostecipada());
                
        }
    }
    
    private Double rentabilidadePrevidenciaPGBL(Double valorInvestido, Double rentabilidadeAoMes, int mes, InvestimentoPrevidencia.TRIBUTACAO_PREVIDENCIA tributacao, Double taxaDeCarregamentoAntecipada, Double taxaDeCarregamentoPostecipada) {
        
        Double mesRentabilidade = new Double(mes);
        
        Double resultado;
        Double rendimento;
        Double percentualTributacao;
        Double valorTributado;
        
        if (taxaDeCarregamentoAntecipada < 0 || taxaDeCarregamentoPostecipada < 0) {
            throw new IllegalArgumentException("Taxas de carregamento devem ser valores positivos!");
        }
        
        // descontando possível taxa de carregamento antecipada
        valorInvestido = valorInvestido * (1-taxaDeCarregamentoAntecipada/100);
        
        // Calculando o rendimento bruto
        rendimento = valorInvestido * Math.pow((1 +(rentabilidadeAoMes/100)), mesRentabilidade);
        rendimento = rendimento - valorInvestido;
        
        // descontando possível taxa de carregamento postecipada
        valorInvestido = valorInvestido * (1-taxaDeCarregamentoPostecipada/100);
        
        // Deduzindo imposto de renda
        valorTributado = valorInvestido + rendimento;
        percentualTributacao = this.getPercentualTributacaoIRPrevidencia(tributacao, mes);
        resultado = deduzPercentual(valorTributado, percentualTributacao);
        
        return resultado;        
    }
    
    private Double rentabilidadePrevidenciaVGBL(Double valorInvestido, Double rentabilidadeAoMes, int mes, InvestimentoPrevidencia.TRIBUTACAO_PREVIDENCIA tributacao, Double taxaDeCarregamentoAntecipada, Double taxaDeCarregamentoPostecipada) {
        Double mesRentabilidade = new Double(mes);
        
        Double resultado;
        Double rendimento;
        Double percentualTributacao;
        Double valorTributado;
        
        if (taxaDeCarregamentoAntecipada < 0.0 || taxaDeCarregamentoPostecipada < 0.0) {
            throw new IllegalArgumentException("Taxas de carregamento devem ser valores positivos!");
        }
        
        // descontando possível taxa de carregamento antecipada
        valorInvestido = valorInvestido * (1-taxaDeCarregamentoAntecipada/100);
        
        // Calculando o rendimento bruto
        rendimento = valorInvestido * Math.pow((1 + (rentabilidadeAoMes/100)), mesRentabilidade);
        rendimento = rendimento - valorInvestido;
        
        // descontando possível taxa de carregamento postecipada
        valorInvestido = valorInvestido * (1-taxaDeCarregamentoPostecipada/100);
        
        // Deduzindo imposto de renda
        valorTributado = rendimento;
        percentualTributacao = this.getPercentualTributacaoIRPrevidencia(tributacao, mes);
        resultado = valorInvestido + deduzPercentual(valorTributado, percentualTributacao);
        
        
        return resultado;  
    }
    
    /**
     * Retorna o percentual de tributação de IR sobre o investimento em previdência
     * de acordo com o tipo de tributação (progressiva/regressiva) e o periodo de
     * duração do investimento em meses.
     * 
     * @param tributacao:   InvestimentoPrevidencia.TRIBUTACAO_PREVIDENCIA.PROGRESSIVA ou InvestimentoPrevidencia.TRIBUTACAO_PREVIDENCIA.REGRESSIVA
     * @param qtdMeses:     Quantidade de meses do investimentos.
     * 
     * @return              Percentual a ser tributado do investimento em previdência.
     */
    private Double getPercentualTributacaoIRPrevidencia(InvestimentoPrevidencia.TRIBUTACAO_PREVIDENCIA tributacao, int qtdMeses){
        
        Double retorno = null;
        
        if(tributacao == null) {
            throw new IllegalArgumentException("Argumento com o tipo de tributação da previdência nulo.");
        } else switch (tributacao) {
            case PROGRESSIVA:
                retorno = 15.00;
                break;
            case REGRESSIVA:
                if (qtdMeses > 120)     // Acima de 10 anos - 10%
                    retorno = 10.00;
                else if (qtdMeses > 96) // De 8 a 10 anos - 15%
                    retorno = 15.00;
                else if (qtdMeses > 72) // De 6 a 8 anos - 20%
                    retorno = 20.00;
                else if (qtdMeses > 48) // De 4 a 6 anos - 25%
                    retorno = 25.00;
                else if (qtdMeses > 24) // De 2 a 4 anos - 30%
                    retorno = 30.00;
                else                    // Até 2 anos - 35%
                    retorno = 35.00;
                break;
        }
        
        return retorno;
    }
    
    
    
//------------------------------------------ CALCULOS REFERENTES A TESOURO IPCA -----------------------------------------------------------------
// OBSERVAÇÃO: calculos para tesouro Selic e IPCA não possui a analize de marcação a mercado.
    private RentabilidadeInvestimento calculaRentabilidadeTesouroIPCA(InvestimentoTesouroIPCA investimentoTesouroIPCA, final int prazo, final int intervaloEmMesCalculo, final Double valorInvestido, Calendar mesAnoAplicacao) {

        RentabilidadeInvestimento resultado = new RentabilidadeInvestimento(investimentoTesouroIPCA);

        Double rentabilidadeA;
        Double rentabilidadeB;
        Double rentabilidadeC;
        Double rentabilidadeD;
        Double rentabilidadeE;
        Double rentabilidadeF;

        int qtdMesesAteVencimentos = getDiferencaEmMesesEntreDatas(mesAnoAplicacao, investimentoTesouroIPCA.getVencimento());
        
        for (int i = intervaloEmMesCalculo; i <= prazo; i = i + intervaloEmMesCalculo) {
            // levando em consideração o prazo de investimento da aplicação (e não apenas o da simulação).
            if ((i <= qtdMesesAteVencimentos && CONSIDERAR_PRAZO_VENCIMENTO_APLICACOES) || CONSIDERAR_PRAZO_VENCIMENTO_APLICACOES == false){
                rentabilidadeA = this.rentabilidadeTesouroIPCA(valorInvestido, investimentoTesouroIPCA.getRentabilidadeAdicionalContratada(), i, this.indiceIPCA.getValorCenario1());
                rentabilidadeB = this.rentabilidadeTesouroIPCA(valorInvestido, investimentoTesouroIPCA.getRentabilidadeAdicionalContratada(), i, this.indiceIPCA.getValorCenario2());
                rentabilidadeC = this.rentabilidadeTesouroIPCA(valorInvestido, investimentoTesouroIPCA.getRentabilidadeAdicionalContratada(), i, this.indiceIPCA.getValorCenario3());
                rentabilidadeD = this.rentabilidadeTesouroIPCA(valorInvestido, investimentoTesouroIPCA.getRentabilidadeAdicionalContratada(), i, this.indiceIPCA.getValorCenario4());
                rentabilidadeE = this.rentabilidadeTesouroIPCA(valorInvestido, investimentoTesouroIPCA.getRentabilidadeAdicionalContratada(), i, this.indiceIPCA.getValorCenario5());
                rentabilidadeF = this.rentabilidadeTesouroIPCA(valorInvestido, investimentoTesouroIPCA.getRentabilidadeAdicionalContratada(), i, this.indiceIPCA.getValorCenario6());
            } else {
                rentabilidadeA = null;
                rentabilidadeB = null;
                rentabilidadeC = null;
                rentabilidadeD = null;
                rentabilidadeE = null;
                rentabilidadeF = null;
            }
            
            resultado.addRentabilidadeMes(rentabilidadeA, rentabilidadeB, rentabilidadeC, rentabilidadeD, rentabilidadeE, rentabilidadeF);
        }

        return resultado;
    }

    private Double rentabilidadeTesouroIPCA(Double valorInvestido, Double percentualAdicionalContratado, int mes, Double valorIPCA) {

        Double mesRentabilidade = new Double(mes);
        Double resultado;
        Double rendimento;

        Double taxaAoMes = convertePercentualAnualParaMensal(valorIPCA+percentualAdicionalContratado);

        // Calculando rentabilidade bruta
        resultado = valorInvestido * Math.pow((1 + ((taxaAoMes)/ 100)), mesRentabilidade);
        
        // Deduzindo imposto de renda
        rendimento = resultado - valorInvestido;
        if(rendimento > 0) { // caso o rendimento da aplicação seja positivo
            rendimento = deduzTributacaoIRTesouroIPCA(rendimento, mes);
        }

        return valorInvestido + rendimento;
    }

    private Double deduzTributacaoIRTesouroIPCA(Double montante, int qtdMeses) {
        return deduzTriburacaoIRRendaFixaBase(montante, qtdMeses);
    }

    
    
    
//------------------------------------------ CALCULOS REFERENTES A TESOURO SELIC -----------------------------------------------------------------
// OBSERVAÇÃO: calculos para tesouro Selic e IPCA não possui a analize de marcação a mercado.
    private RentabilidadeInvestimento calculaRentabilidadeTesouroSelic(InvestimentoTesouroSelic investimentoTesouroSelic, final int prazo, final int intervaloEmMesCalculo, final Double valorInvestido, Calendar mesAnoAplicacao) {

        RentabilidadeInvestimento resultado = new RentabilidadeInvestimento(investimentoTesouroSelic);

        Double rentabilidadeA;
        Double rentabilidadeB;
        Double rentabilidadeC;
        Double rentabilidadeD;
        Double rentabilidadeE;
        Double rentabilidadeF;
        
        int qtdMesesAteVencimentos = getDiferencaEmMesesEntreDatas(mesAnoAplicacao, investimentoTesouroSelic.getVencimento());
        
        for (int i = intervaloEmMesCalculo; i <= prazo; i = i + intervaloEmMesCalculo) {
            // levando em consideração o prazo de investimento da aplicação (e não apenas o da simulação).
            if ((i <= qtdMesesAteVencimentos && CONSIDERAR_PRAZO_VENCIMENTO_APLICACOES) || CONSIDERAR_PRAZO_VENCIMENTO_APLICACOES == false){
            rentabilidadeA = this.rentabilidadeTesouroSelic(valorInvestido, investimentoTesouroSelic.getRendimentoAdicional(), i, this.indiceSelic.getValorCenario1());
            rentabilidadeB = this.rentabilidadeTesouroSelic(valorInvestido, investimentoTesouroSelic.getRendimentoAdicional(), i, this.indiceSelic.getValorCenario2());
            rentabilidadeC = this.rentabilidadeTesouroSelic(valorInvestido, investimentoTesouroSelic.getRendimentoAdicional(), i, this.indiceSelic.getValorCenario3());
            rentabilidadeD = this.rentabilidadeTesouroSelic(valorInvestido, investimentoTesouroSelic.getRendimentoAdicional(), i, this.indiceSelic.getValorCenario4());
            rentabilidadeE = this.rentabilidadeTesouroSelic(valorInvestido, investimentoTesouroSelic.getRendimentoAdicional(), i, this.indiceSelic.getValorCenario5());
            rentabilidadeF = this.rentabilidadeTesouroSelic(valorInvestido, investimentoTesouroSelic.getRendimentoAdicional(), i, this.indiceSelic.getValorCenario6());
            } else {
            rentabilidadeA = null;
            rentabilidadeB = null;
            rentabilidadeC = null;
            rentabilidadeD = null;
            rentabilidadeE = null;
            rentabilidadeF = null;
            }
            
            resultado.addRentabilidadeMes(rentabilidadeA, rentabilidadeB, rentabilidadeC, rentabilidadeD, rentabilidadeE, rentabilidadeF);
        }

        return resultado;
    }

    private Double rentabilidadeTesouroSelic(Double valorInvestido, Double taxaAdicionalAnual, int mes, Double valorSelic) {

        Double mesRentabilidade = new Double(mes);
        Double resultado;
        Double rendimento;
        
        Double rentabilidadeAnual = valorSelic + taxaAdicionalAnual;
                
        // Calculando rentabilidade bruta
        Double rentabilidadeAoMes = convertePercentualAnualParaMensal(rentabilidadeAnual);
        resultado = valorInvestido * Math.pow((1 + (rentabilidadeAoMes / 100)), mesRentabilidade);

        // Deduzindo imposto de renda
        rendimento = resultado - valorInvestido;
        if(rendimento > 0) { // caso o rendimento da aplicação seja positivo
            rendimento = deduzTributacaoIRTesouroSelic(rendimento, mes);
        }

        return valorInvestido + rendimento;
    }

    private Double deduzTributacaoIRTesouroSelic(Double montante, int qtdMeses) {
        return deduzTriburacaoIRRendaFixaBase(montante, qtdMeses);
    }

    
    
    
//------------------------------------------ CALCULOS REFERENTES A Poupanca ----------------------------------------------------------------------------------
    private RentabilidadeInvestimento calculaRentabilidadePoupanca(InvestimentoPoupanca investimentoPoupanca, final int prazo, final int intervaloEmMesCalculo, final Double valorInvestido) {

        RentabilidadeInvestimento resultado = new RentabilidadeInvestimento(investimentoPoupanca);

        Double rentabilidadeA;
        Double rentabilidadeB;
        Double rentabilidadeC;
        Double rentabilidadeD;
        Double rentabilidadeE;
        Double rentabilidadeF;

        for (int i = intervaloEmMesCalculo; i <= prazo; i = i + intervaloEmMesCalculo) {
            rentabilidadeA = rentabilidadePoupanca(valorInvestido, CENARIO.A, i);
            rentabilidadeB = rentabilidadePoupanca(valorInvestido, CENARIO.B, i);
            rentabilidadeC = rentabilidadePoupanca(valorInvestido, CENARIO.C, i);
            rentabilidadeD = rentabilidadePoupanca(valorInvestido, CENARIO.D, i);
            rentabilidadeE = rentabilidadePoupanca(valorInvestido, CENARIO.E, i);
            rentabilidadeF = rentabilidadePoupanca(valorInvestido, CENARIO.F, i);

            resultado.addRentabilidadeMes(rentabilidadeA, rentabilidadeB, rentabilidadeC, rentabilidadeD, rentabilidadeE, rentabilidadeF);
        }

        return resultado;
    }

    private Double rentabilidadePoupanca(Double valorInvestido, CENARIO cenario, int mes) {

        Double resultado = null;

        if (null != cenario) {
            switch (cenario) {
                case A:
                    resultado = rentabilidadePoupanca(valorInvestido, this.indiceSelic.getValorCenario1(), this.indiceTR.getValorCenario1(), mes);
                    break;
                case B:
                    resultado = rentabilidadePoupanca(valorInvestido, this.indiceSelic.getValorCenario2(), this.indiceTR.getValorCenario2(), mes);
                    break;
                case C:
                    resultado = rentabilidadePoupanca(valorInvestido, this.indiceSelic.getValorCenario3(), this.indiceTR.getValorCenario3(), mes);
                    break;
                case D:
                    resultado = rentabilidadePoupanca(valorInvestido, this.indiceSelic.getValorCenario4(), this.indiceTR.getValorCenario4(), mes);
                    break;
                case E:
                    resultado = rentabilidadePoupanca(valorInvestido, this.indiceSelic.getValorCenario5(), this.indiceTR.getValorCenario5(), mes);
                    break;
                case F:
                    resultado = rentabilidadePoupanca(valorInvestido, this.indiceSelic.getValorCenario6(), this.indiceTR.getValorCenario6(), mes);
                    break;
                default:
                    break;
            }
        }

        return resultado;
    }

    private Double rentabilidadePoupanca(Double valorInvestido, Double Selic, Double TRanual, int mes) {

        Double mesRentabilidade = new Double(mes);

        Double resultado;
        Double rentabilidadeAoMes;

        Double selicAoMes = convertePercentualAnualParaMensal(Selic);
        Double TR = convertePercentualAnualParaMensal(TRanual);

        // Calculando a rentabilidade
        if (Selic > 8.5) {
            rentabilidadeAoMes = 0.5 + TR;
        } else {
            rentabilidadeAoMes = 0.7 * selicAoMes + TR;
        }

        resultado = valorInvestido * Math.pow((1 + (rentabilidadeAoMes / 100)), mesRentabilidade);

        return resultado;
    }

    
    
    
//------------------------------------------ CALCULOS REFERENTES A RDB ----------------------------------------------------------------------------------
    private RentabilidadeInvestimento calculaRentabilidadeRDB(InvestimentoRDB investimentoRDB, final int prazo, final int intervaloEmMesCalculo, final Double valorInvestido, Calendar mesAnoAplicacao) {

        RentabilidadeInvestimento rentabilidadeInvestimento = new RentabilidadeInvestimento(investimentoRDB);

        if (investimentoRDB.isPrefixado()) {
            rentabilidadeInvestimento = rentabilidadeRDBPrefixado(investimentoRDB, prazo, intervaloEmMesCalculo, valorInvestido, mesAnoAplicacao);
        } else if (investimentoRDB.isPosFixado()) {
            rentabilidadeInvestimento = rentabilidadeRDBPosfixada(investimentoRDB, prazo, intervaloEmMesCalculo, valorInvestido, mesAnoAplicacao);
        }

        return rentabilidadeInvestimento;
    }

    private RentabilidadeInvestimento rentabilidadeRDBPrefixado(InvestimentoRDB investimentoRDB, final int prazo, final int intervaloEmMesCalculo, final Double valorInvestido, Calendar mesAnoAplicacao) {

        RentabilidadeInvestimento resultado = new RentabilidadeInvestimento(investimentoRDB);

        Double rentabilidadeA;
        Double rentabilidadeB;
        Double rentabilidadeC;
        Double rentabilidadeD;
        Double rentabilidadeE;
        Double rentabilidadeF;
               
        // passando a taxa de rentabilidade de anual para mensal
        Double taxaAoMesCenarioA = convertePercentualAnualParaMensal(investimentoRDB.getValorCenario1());
        Double taxaAoMesCenarioB = convertePercentualAnualParaMensal(investimentoRDB.getValorCenario2());
        Double taxaAoMesCenarioC = convertePercentualAnualParaMensal(investimentoRDB.getValorCenario3());
        Double taxaAoMesCenarioD = convertePercentualAnualParaMensal(investimentoRDB.getValorCenario4());
        Double taxaAoMesCenarioE = convertePercentualAnualParaMensal(investimentoRDB.getValorCenario5());
        Double taxaAoMesCenarioF = convertePercentualAnualParaMensal(investimentoRDB.getValorCenario6());

        int qtdMesesAteVencimentos = getDiferencaEmMesesIntervaloDeDias(mesAnoAplicacao, investimentoRDB.getPrazoDoInvestimento());
        
        for (int i = intervaloEmMesCalculo; i <= prazo; i = i + intervaloEmMesCalculo) {
            
            // levando em consideração o prazo de investimento da aplicação (e não apenas o da simulação).
            if ((i <= qtdMesesAteVencimentos && CONSIDERAR_PRAZO_VENCIMENTO_APLICACOES) || CONSIDERAR_PRAZO_VENCIMENTO_APLICACOES == false){   
                rentabilidadeA = rentabilidadeRDBPrefixado(valorInvestido, taxaAoMesCenarioA, i);
                rentabilidadeB = rentabilidadeRDBPrefixado(valorInvestido, taxaAoMesCenarioB, i);
                rentabilidadeC = rentabilidadeRDBPrefixado(valorInvestido, taxaAoMesCenarioC, i);
                rentabilidadeD = rentabilidadeRDBPrefixado(valorInvestido, taxaAoMesCenarioD, i);
                rentabilidadeE = rentabilidadeRDBPrefixado(valorInvestido, taxaAoMesCenarioE, i);
                rentabilidadeF = rentabilidadeRDBPrefixado(valorInvestido, taxaAoMesCenarioF, i);
            } else {
                rentabilidadeA = null;
                rentabilidadeB = null;
                rentabilidadeC = null;
                rentabilidadeD = null;
                rentabilidadeE = null;
                rentabilidadeF = null;
            }
            resultado.addRentabilidadeMes(rentabilidadeA, rentabilidadeB, rentabilidadeC, rentabilidadeD, rentabilidadeE, rentabilidadeF);
        }

        return resultado;
    }

    private Double rentabilidadeRDBPrefixado(Double valorInvestido, Double rentabilidadeAoMes, int mes) {

        Double mesRentabilidade = new Double(mes);

        Double resultado;
        Double rendimento;

        // Calculando rentabilidade bruta
        resultado = valorInvestido * Math.pow((1 + (rentabilidadeAoMes / 100)), mesRentabilidade);

        // Deduzindo imposto de renda
        rendimento = resultado - valorInvestido;
        if(rendimento > 0) { // caso o rendimento da aplicação seja positivo
            rendimento = deduzTributacaoIRRDB(rendimento, mes);
        }

        return valorInvestido + rendimento;
    }

    private RentabilidadeInvestimento rentabilidadeRDBPosfixada(InvestimentoRDB investimentoRDB, final int prazo, final int intervaloEmMesCalculo, final Double valorInvestido, Calendar mesAnoAplicacao) {

        RentabilidadeInvestimento resultado = new RentabilidadeInvestimento(investimentoRDB);

        Double rentabilidadeA;
        Double rentabilidadeB;
        Double rentabilidadeC;
        Double rentabilidadeD;
        Double rentabilidadeE;
        Double rentabilidadeF;

        int qtdMesesAteVencimentos = getDiferencaEmMesesIntervaloDeDias(mesAnoAplicacao, investimentoRDB.getPrazoDoInvestimento());
        
        for (int i = intervaloEmMesCalculo; i <= prazo; i = i + intervaloEmMesCalculo) {
            // levando em consideração o prazo de investimento da aplicação (e não apenas o da simulação).
            if ((i <= qtdMesesAteVencimentos && CONSIDERAR_PRAZO_VENCIMENTO_APLICACOES) || CONSIDERAR_PRAZO_VENCIMENTO_APLICACOES == false){    
                rentabilidadeA = this.rentabilidadeRDBPosfixada(valorInvestido, i, investimentoRDB.getPercentualCDIContratado(), this.indiceCDI.getValorCenario1());
                rentabilidadeB = this.rentabilidadeRDBPosfixada(valorInvestido, i, investimentoRDB.getPercentualCDIContratado(), this.indiceCDI.getValorCenario2());
                rentabilidadeC = this.rentabilidadeRDBPosfixada(valorInvestido, i, investimentoRDB.getPercentualCDIContratado(), this.indiceCDI.getValorCenario3());
                rentabilidadeD = this.rentabilidadeRDBPosfixada(valorInvestido, i, investimentoRDB.getPercentualCDIContratado(), this.indiceCDI.getValorCenario4());
                rentabilidadeE = this.rentabilidadeRDBPosfixada(valorInvestido, i, investimentoRDB.getPercentualCDIContratado(), this.indiceCDI.getValorCenario5());
                rentabilidadeF = this.rentabilidadeRDBPosfixada(valorInvestido, i, investimentoRDB.getPercentualCDIContratado(), this.indiceCDI.getValorCenario6());
            } else {
                rentabilidadeA = null;
                rentabilidadeB = null;
                rentabilidadeC = null;
                rentabilidadeD = null;
                rentabilidadeE = null;
                rentabilidadeF = null;
            }
            resultado.addRentabilidadeMes(rentabilidadeA, rentabilidadeB, rentabilidadeC, rentabilidadeD, rentabilidadeE, rentabilidadeF);
        }

        return resultado;
    }

    private Double rentabilidadeRDBPosfixada(Double valorInvestido, int mes, Double percentualCDIContratado, Double valorCDI) {

        Double mesRentabilidade = new Double(mes);
        Double resultado;
        Double rendimento;

        Double cdiAoMes = convertePercentualAnualParaMensal(valorCDI);

        // Calculando rentabilidade bruta
        Double rentabilidadeAoMes = percentualCDIContratado / 100 * cdiAoMes;
        resultado = valorInvestido * Math.pow((1 + (rentabilidadeAoMes / 100)), mesRentabilidade);

        // Deduzindo imposto de renda
        rendimento = resultado - valorInvestido;
        if(rendimento > 0) { // caso o rendimento da aplicação seja positivo
            rendimento = deduzTributacaoIRRDB(rendimento, mes);
        }

        return valorInvestido + rendimento;
    }

    private Double deduzTributacaoIRRDB(Double montante, int qtdMeses) {
        return deduzTriburacaoIRRendaFixaBase(montante, qtdMeses);
    }

    
    
    
//------------------------------------------ CALCULOS REFERENTES A CDB ----------------------------------------------------------------------------------
    private RentabilidadeInvestimento calculaRentabilidadeCDB(InvestimentoCDB investimentoCDB, final int prazo, final int intervaloEmMesCalculo, final Double valorInvestido, Calendar mesAnoAplicacao) {

        RentabilidadeInvestimento rentabilidadeInvestimento = new RentabilidadeInvestimento(investimentoCDB);

        if (investimentoCDB.isPrefixado()) {
            rentabilidadeInvestimento = rentabilidadeCDBPrefixado(investimentoCDB, prazo, intervaloEmMesCalculo, valorInvestido, mesAnoAplicacao);
        } else if (investimentoCDB.isPosFixado()) {
            rentabilidadeInvestimento = rentabilidadeCDBPosfixada(investimentoCDB, prazo, intervaloEmMesCalculo, valorInvestido, mesAnoAplicacao);
        }

        return rentabilidadeInvestimento;
    }

    private RentabilidadeInvestimento rentabilidadeCDBPrefixado(InvestimentoCDB investimentoCDB, final int prazo, final int intervaloEmMesCalculo, final Double valorInvestido, Calendar mesAnoAplicacao) {

        RentabilidadeInvestimento resultado = new RentabilidadeInvestimento(investimentoCDB);

        Double rentabilidadeA;
        Double rentabilidadeB;
        Double rentabilidadeC;
        Double rentabilidadeD;
        Double rentabilidadeE;
        Double rentabilidadeF;
        
        // passando a taxa de rentabilidade de anual para mensal
        Double taxaAoMesCenarioA = convertePercentualAnualParaMensal(investimentoCDB.getValorCenario1());
        Double taxaAoMesCenarioB = convertePercentualAnualParaMensal(investimentoCDB.getValorCenario2());
        Double taxaAoMesCenarioC = convertePercentualAnualParaMensal(investimentoCDB.getValorCenario3());
        Double taxaAoMesCenarioD = convertePercentualAnualParaMensal(investimentoCDB.getValorCenario4());
        Double taxaAoMesCenarioE = convertePercentualAnualParaMensal(investimentoCDB.getValorCenario5());
        Double taxaAoMesCenarioF = convertePercentualAnualParaMensal(investimentoCDB.getValorCenario6());
        
        int qtdMesesAteVencimentos = getDiferencaEmMesesIntervaloDeDias(mesAnoAplicacao, investimentoCDB.getPrazoDoInvestimento());

        for (int i = intervaloEmMesCalculo; i <= prazo; i = i + intervaloEmMesCalculo) {
            // levando em consideração o prazo de investimento da aplicação (e não apenas o da simulação).
            if ((i <= qtdMesesAteVencimentos && CONSIDERAR_PRAZO_VENCIMENTO_APLICACOES) || CONSIDERAR_PRAZO_VENCIMENTO_APLICACOES == false) {
                rentabilidadeA = rentabilidadeCDBPrefixado(valorInvestido, taxaAoMesCenarioA, i);
                rentabilidadeB = rentabilidadeCDBPrefixado(valorInvestido, taxaAoMesCenarioB, i);
                rentabilidadeC = rentabilidadeCDBPrefixado(valorInvestido, taxaAoMesCenarioC, i);
                rentabilidadeD = rentabilidadeCDBPrefixado(valorInvestido, taxaAoMesCenarioD, i);
                rentabilidadeE = rentabilidadeCDBPrefixado(valorInvestido, taxaAoMesCenarioE, i);
                rentabilidadeF = rentabilidadeCDBPrefixado(valorInvestido, taxaAoMesCenarioF, i);
            } else {
                rentabilidadeA = null;
                rentabilidadeB = null;
                rentabilidadeC = null;
                rentabilidadeD = null;
                rentabilidadeE = null;
                rentabilidadeF = null;
            }
            resultado.addRentabilidadeMes(rentabilidadeA, rentabilidadeB, rentabilidadeC, rentabilidadeD, rentabilidadeE, rentabilidadeF);
        }

        return resultado;
    }

    private Double rentabilidadeCDBPrefixado(Double valorInvestido, Double rentabilidadeAoMes, int mes) {

        Double mesRentabilidade = new Double(mes);

        Double resultado;
        Double rendimento;

        // Calculando rentabilidade bruta
        resultado = valorInvestido * Math.pow((1 + (rentabilidadeAoMes / 100)), mesRentabilidade);

        // Deduzindo imposto de renda
        rendimento = resultado - valorInvestido;
        if(rendimento > 0) { // caso o rendimento da aplicação seja positivo
            rendimento = deduzTributacaoIRCDB(rendimento, mes);
        }

        return valorInvestido + rendimento;
    }

    private RentabilidadeInvestimento rentabilidadeCDBPosfixada(InvestimentoCDB investimentoCDB, final int prazo, final int intervaloEmMesCalculo, final Double valorInvestido, Calendar mesAnoAplicacao) {

        RentabilidadeInvestimento resultado = new RentabilidadeInvestimento(investimentoCDB);

        Double rentabilidadeA;
        Double rentabilidadeB;
        Double rentabilidadeC;
        Double rentabilidadeD;
        Double rentabilidadeE;
        Double rentabilidadeF;

        int qtdMesesAteVencimentos = getDiferencaEmMesesIntervaloDeDias(mesAnoAplicacao, investimentoCDB.getPrazoDoInvestimento());
        
        for (int i = intervaloEmMesCalculo; i <= prazo; i = i + intervaloEmMesCalculo) {
            
            // levando em consideração o prazo de investimento da aplicação (e não apenas o da simulação).
            if ((i <= qtdMesesAteVencimentos && CONSIDERAR_PRAZO_VENCIMENTO_APLICACOES) || CONSIDERAR_PRAZO_VENCIMENTO_APLICACOES == false){
                rentabilidadeA = this.rentabilidadeCDBPosfixada(valorInvestido, i, investimentoCDB.getPercentualCDIContratado(), this.indiceCDI.getValorCenario1());
                rentabilidadeB = this.rentabilidadeCDBPosfixada(valorInvestido, i, investimentoCDB.getPercentualCDIContratado(), this.indiceCDI.getValorCenario2());
                rentabilidadeC = this.rentabilidadeCDBPosfixada(valorInvestido, i, investimentoCDB.getPercentualCDIContratado(), this.indiceCDI.getValorCenario3());
                rentabilidadeD = this.rentabilidadeCDBPosfixada(valorInvestido, i, investimentoCDB.getPercentualCDIContratado(), this.indiceCDI.getValorCenario4());
                rentabilidadeE = this.rentabilidadeCDBPosfixada(valorInvestido, i, investimentoCDB.getPercentualCDIContratado(), this.indiceCDI.getValorCenario5());
                rentabilidadeF = this.rentabilidadeCDBPosfixada(valorInvestido, i, investimentoCDB.getPercentualCDIContratado(), this.indiceCDI.getValorCenario6());
            } else {
                rentabilidadeA = null;
                rentabilidadeB = null;
                rentabilidadeC = null;
                rentabilidadeD = null;
                rentabilidadeE = null;
                rentabilidadeF = null;
            }
            
            resultado.addRentabilidadeMes(rentabilidadeA, rentabilidadeB, rentabilidadeC, rentabilidadeD, rentabilidadeE, rentabilidadeF);
        }

        return resultado;
    }

    private Double rentabilidadeCDBPosfixada(Double valorInvestido, int mes, Double percentualCDIContratado, Double valorCDI) {

        Double mesRentabilidade = new Double(mes);
        Double resultado;
        Double rendimento;

        Double cdiAoMes = convertePercentualAnualParaMensal(valorCDI);

        // Calculando rentabilidade bruta
        Double rentabilidadeAoMes = percentualCDIContratado / 100 * cdiAoMes;
        resultado = valorInvestido * Math.pow((1 + (rentabilidadeAoMes / 100)), mesRentabilidade);

        // Deduzindo imposto de renda
        rendimento = resultado - valorInvestido;
        if(rendimento > 0) { // caso o rendimento da aplicação seja positivo
            rendimento = deduzTributacaoIRCDB(rendimento, mes);
        }

        return valorInvestido + rendimento;
    }

    private Double deduzTributacaoIRCDB(Double montante, int qtdMeses) {
        return deduzTriburacaoIRRendaFixaBase(montante, qtdMeses);
    }

    
    
    
//------------------------------------------ CALCULOS REFERENTES A LCA ----------------------------------------------------------------------------------
    private RentabilidadeInvestimento calculaRentabilidadeLCA(InvestimentoLCA investimentoLCA, final int prazo, final int intervaloEmMesCalculo, final Double valorInvestido, Calendar mesAnoAplicacao) {

        RentabilidadeInvestimento rentabilidadeInvestimento = new RentabilidadeInvestimento(investimentoLCA);

        if (investimentoLCA.isPrefixado()) {
            rentabilidadeInvestimento = rentabilidadeLCAPrefixada(investimentoLCA, prazo, intervaloEmMesCalculo, valorInvestido, mesAnoAplicacao);
        } else if (investimentoLCA.isPosFixado()) {
            rentabilidadeInvestimento = rentabilidadeLCAPosfixada(investimentoLCA, prazo, intervaloEmMesCalculo, valorInvestido, mesAnoAplicacao);
        }

        return rentabilidadeInvestimento;
    }

    private RentabilidadeInvestimento rentabilidadeLCAPrefixada(InvestimentoLCA investimentoLCA, final int prazo, final int intervaloEmMesCalculo, final Double valorInvestido, Calendar mesAnoAplicacao) {

        RentabilidadeInvestimento resultado = new RentabilidadeInvestimento(investimentoLCA);

        Double rentabilidadeA;
        Double rentabilidadeB;
        Double rentabilidadeC;
        Double rentabilidadeD;
        Double rentabilidadeE;
        Double rentabilidadeF;
        
        // passando a taxa de rentabilidade de anual para mensal
        Double taxaAoMesCenarioA = convertePercentualAnualParaMensal(investimentoLCA.getValorCenario1());
        Double taxaAoMesCenarioB = convertePercentualAnualParaMensal(investimentoLCA.getValorCenario2());
        Double taxaAoMesCenarioC = convertePercentualAnualParaMensal(investimentoLCA.getValorCenario3());
        Double taxaAoMesCenarioD = convertePercentualAnualParaMensal(investimentoLCA.getValorCenario4());
        Double taxaAoMesCenarioE = convertePercentualAnualParaMensal(investimentoLCA.getValorCenario5());
        Double taxaAoMesCenarioF = convertePercentualAnualParaMensal(investimentoLCA.getValorCenario6());

        int qtdMesesAteVencimentos = getDiferencaEmMesesIntervaloDeDias(mesAnoAplicacao, investimentoLCA.getPrazoDoInvestimento());
        
        for (int i = intervaloEmMesCalculo; i <= prazo; i = i + intervaloEmMesCalculo) {
            // levando em consideração o prazo de investimento da aplicação (e não apenas o da simulação).
            if ((i <= qtdMesesAteVencimentos && CONSIDERAR_PRAZO_VENCIMENTO_APLICACOES) || CONSIDERAR_PRAZO_VENCIMENTO_APLICACOES == false){
                rentabilidadeA = rentabilidadeLCAPrefixada(valorInvestido, taxaAoMesCenarioA, i);
                rentabilidadeB = rentabilidadeLCAPrefixada(valorInvestido, taxaAoMesCenarioB, i);
                rentabilidadeC = rentabilidadeLCAPrefixada(valorInvestido, taxaAoMesCenarioC, i);
                rentabilidadeD = rentabilidadeLCAPrefixada(valorInvestido, taxaAoMesCenarioD, i);
                rentabilidadeE = rentabilidadeLCAPrefixada(valorInvestido, taxaAoMesCenarioE, i);
                rentabilidadeF = rentabilidadeLCAPrefixada(valorInvestido, taxaAoMesCenarioF, i);
            } else {
                rentabilidadeA = null;
                rentabilidadeB = null;
                rentabilidadeC = null;
                rentabilidadeD = null;
                rentabilidadeE = null;
                rentabilidadeF = null;
            }

            resultado.addRentabilidadeMes(rentabilidadeA, rentabilidadeB, rentabilidadeC, rentabilidadeD, rentabilidadeE, rentabilidadeF);
        }

        return resultado;
    }

    private Double rentabilidadeLCAPrefixada(Double valorInvestido, Double rentabilidadeAoMes, int mes) {

        Double mesRentabilidade = new Double(mes);

        Double resultado;

        resultado = valorInvestido * Math.pow((1 + (rentabilidadeAoMes / 100)), mesRentabilidade);

        return resultado;
    }

    private RentabilidadeInvestimento rentabilidadeLCAPosfixada(InvestimentoLCA investimentoLCA, final int prazo, final int intervaloEmMesCalculo, final Double valorInvestido, Calendar mesAnoAplicacao) {

        RentabilidadeInvestimento resultado = new RentabilidadeInvestimento(investimentoLCA);

        Double rentabilidadeA;
        Double rentabilidadeB;
        Double rentabilidadeC;
        Double rentabilidadeD;
        Double rentabilidadeE;
        Double rentabilidadeF;

        int qtdMesesAteVencimentos = getDiferencaEmMesesIntervaloDeDias(mesAnoAplicacao, investimentoLCA.getPrazoDoInvestimento());
        
        for (int i = intervaloEmMesCalculo; i <= prazo; i = i + intervaloEmMesCalculo) {
            // levando em consideração o prazo de investimento da aplicação (e não apenas o da simulação).
            if ((i <= qtdMesesAteVencimentos && CONSIDERAR_PRAZO_VENCIMENTO_APLICACOES) || CONSIDERAR_PRAZO_VENCIMENTO_APLICACOES == false){
                rentabilidadeA = this.rentabilidadeLCAPosfixada(valorInvestido, i, investimentoLCA.getPercentualCDIContratado(), this.indiceCDI.getValorCenario1());
                rentabilidadeB = this.rentabilidadeLCAPosfixada(valorInvestido, i, investimentoLCA.getPercentualCDIContratado(), this.indiceCDI.getValorCenario2());
                rentabilidadeC = this.rentabilidadeLCAPosfixada(valorInvestido, i, investimentoLCA.getPercentualCDIContratado(), this.indiceCDI.getValorCenario3());
                rentabilidadeD = this.rentabilidadeLCAPosfixada(valorInvestido, i, investimentoLCA.getPercentualCDIContratado(), this.indiceCDI.getValorCenario4());
                rentabilidadeE = this.rentabilidadeLCAPosfixada(valorInvestido, i, investimentoLCA.getPercentualCDIContratado(), this.indiceCDI.getValorCenario5());
                rentabilidadeF = this.rentabilidadeLCAPosfixada(valorInvestido, i, investimentoLCA.getPercentualCDIContratado(), this.indiceCDI.getValorCenario6());
            } else {
                rentabilidadeA = null;
                rentabilidadeB = null;
                rentabilidadeC = null;
                rentabilidadeD = null;
                rentabilidadeE = null;
                rentabilidadeF = null;
            }
            
            resultado.addRentabilidadeMes(rentabilidadeA, rentabilidadeB, rentabilidadeC, rentabilidadeD, rentabilidadeE, rentabilidadeF);
        }

        return resultado;
    }

    private Double rentabilidadeLCAPosfixada(Double valorInvestido, int mes, Double percentualCDIContratado, Double valorCDI) {

        Double mesRentabilidade = new Double(mes);
        Double resultado;

        Double cdiAoMes = convertePercentualAnualParaMensal(valorCDI);

        Double rentabilidadeAoMes = percentualCDIContratado / 100 * cdiAoMes;
        resultado = valorInvestido * Math.pow((1 + (rentabilidadeAoMes / 100)), mesRentabilidade);

        return resultado;
    }

    
    
    
//------------------------------------------ CALCULOS REFERENTES A LCI ----------------------------------------------------------------------------------
    private RentabilidadeInvestimento calculaRentabilidadeLCI(InvestimentoLCI investimentoLCI, final int prazo, final int intervaloEmMesCalculo, final Double valorInvestido, Calendar mesAnoAplicacao) {

        RentabilidadeInvestimento rentabilidadeInvestimento = new RentabilidadeInvestimento(investimentoLCI);

        if (investimentoLCI.isPrefixado()) {
            rentabilidadeInvestimento = rentabilidadeLciPrefixada(investimentoLCI, prazo, intervaloEmMesCalculo, valorInvestido, mesAnoAplicacao);
        } else if (investimentoLCI.isPosFixado()) {
            rentabilidadeInvestimento = rentabilidadeLciPosfixada(investimentoLCI, prazo, intervaloEmMesCalculo, valorInvestido, mesAnoAplicacao);
        }

        return rentabilidadeInvestimento;
    }

    private RentabilidadeInvestimento rentabilidadeLciPrefixada(InvestimentoLCI investimentoLCI, final int prazo, final int intervaloEmMesCalculo, final Double valorInvestido, Calendar mesAnoAplicacao) {

        RentabilidadeInvestimento resultado = new RentabilidadeInvestimento(investimentoLCI);

        Double rentabilidadeA;
        Double rentabilidadeB;
        Double rentabilidadeC;
        Double rentabilidadeD;
        Double rentabilidadeE;
        Double rentabilidadeF;

        // passando a taxa de rentabilidade de anual para mensal
        Double taxaAoMesCenarioA = convertePercentualAnualParaMensal(investimentoLCI.getValorCenario1());
        Double taxaAoMesCenarioB = convertePercentualAnualParaMensal(investimentoLCI.getValorCenario2());
        Double taxaAoMesCenarioC = convertePercentualAnualParaMensal(investimentoLCI.getValorCenario3());
        Double taxaAoMesCenarioD = convertePercentualAnualParaMensal(investimentoLCI.getValorCenario4());
        Double taxaAoMesCenarioE = convertePercentualAnualParaMensal(investimentoLCI.getValorCenario5());
        Double taxaAoMesCenarioF = convertePercentualAnualParaMensal(investimentoLCI.getValorCenario6());
        
        int qtdMesesAteVencimentos = getDiferencaEmMesesIntervaloDeDias(mesAnoAplicacao, investimentoLCI.getPrazoDoInvestimento());//
        
        for (int i = intervaloEmMesCalculo; i <= prazo; i = i + intervaloEmMesCalculo) {
            
            if ((i <= qtdMesesAteVencimentos && CONSIDERAR_PRAZO_VENCIMENTO_APLICACOES) || CONSIDERAR_PRAZO_VENCIMENTO_APLICACOES == false){//
                rentabilidadeA = rentabilidadeLciPrefixada(valorInvestido, taxaAoMesCenarioA, i);
                rentabilidadeB = rentabilidadeLciPrefixada(valorInvestido, taxaAoMesCenarioB, i);
                rentabilidadeC = rentabilidadeLciPrefixada(valorInvestido, taxaAoMesCenarioC, i);
                rentabilidadeD = rentabilidadeLciPrefixada(valorInvestido, taxaAoMesCenarioD, i);
                rentabilidadeE = rentabilidadeLciPrefixada(valorInvestido, taxaAoMesCenarioE, i);
                rentabilidadeF = rentabilidadeLciPrefixada(valorInvestido, taxaAoMesCenarioF, i);
            } else {//
                rentabilidadeA = null;
                rentabilidadeB = null;
                rentabilidadeC = null;
                rentabilidadeD = null;
                rentabilidadeE = null;
                rentabilidadeF = null;
            }

            resultado.addRentabilidadeMes(rentabilidadeA, rentabilidadeB, rentabilidadeC, rentabilidadeD, rentabilidadeE, rentabilidadeF);
        }

        return resultado;
    }

    private Double rentabilidadeLciPrefixada(Double valorInvestido, Double rentabilidadeAoMes, int mes) {

        Double mesRentabilidade = new Double(mes);

        Double resultado;

        resultado = valorInvestido * Math.pow((1 + (rentabilidadeAoMes / 100)), mesRentabilidade);

        return resultado;
    }

    private RentabilidadeInvestimento rentabilidadeLciPosfixada(InvestimentoLCI investimentoLCI, final int prazo, final int intervaloEmMesCalculo, final Double valorInvestido, Calendar mesAnoAplicacao) {

        RentabilidadeInvestimento resultado = new RentabilidadeInvestimento(investimentoLCI);

        Double rentabilidadeA;
        Double rentabilidadeB;
        Double rentabilidadeC;
        Double rentabilidadeD;
        Double rentabilidadeE;
        Double rentabilidadeF;

        int qtdMesesAteVencimentos = getDiferencaEmMesesIntervaloDeDias(mesAnoAplicacao, investimentoLCI.getPrazoDoInvestimento());
        
        for (int i = intervaloEmMesCalculo; i <= prazo; i = i + intervaloEmMesCalculo) {
            if ((i <= qtdMesesAteVencimentos && CONSIDERAR_PRAZO_VENCIMENTO_APLICACOES) || CONSIDERAR_PRAZO_VENCIMENTO_APLICACOES == false) {
                rentabilidadeA = this.rentabilidadeLciPosfixada(valorInvestido, i, investimentoLCI.getPercentualCDIContratado(), this.indiceCDI.getValorCenario1());
                rentabilidadeB = this.rentabilidadeLciPosfixada(valorInvestido, i, investimentoLCI.getPercentualCDIContratado(), this.indiceCDI.getValorCenario2());
                rentabilidadeC = this.rentabilidadeLciPosfixada(valorInvestido, i, investimentoLCI.getPercentualCDIContratado(), this.indiceCDI.getValorCenario3());
                rentabilidadeD = this.rentabilidadeLciPosfixada(valorInvestido, i, investimentoLCI.getPercentualCDIContratado(), this.indiceCDI.getValorCenario4());
                rentabilidadeE = this.rentabilidadeLciPosfixada(valorInvestido, i, investimentoLCI.getPercentualCDIContratado(), this.indiceCDI.getValorCenario5());
                rentabilidadeF = this.rentabilidadeLciPosfixada(valorInvestido, i, investimentoLCI.getPercentualCDIContratado(), this.indiceCDI.getValorCenario6());
            } else {
                rentabilidadeA = null;
                rentabilidadeB = null;
                rentabilidadeC = null;
                rentabilidadeD = null;
                rentabilidadeE = null;
                rentabilidadeF = null;
            }
            
            resultado.addRentabilidadeMes(rentabilidadeA, rentabilidadeB, rentabilidadeC, rentabilidadeD, rentabilidadeE, rentabilidadeF);
        }

        return resultado;
    }

    private Double rentabilidadeLciPosfixada(Double valorInvestido, int mes, Double percentualCDIContratado, Double valorCDI) {

        Double mesRentabilidade = new Double(mes);
        Double resultado;

        Double cdiAoMes = convertePercentualAnualParaMensal(valorCDI);

        Double rentabilidadeAoMes = percentualCDIContratado / 100 * cdiAoMes;
        resultado = valorInvestido * Math.pow((1 + (rentabilidadeAoMes / 100)), mesRentabilidade);

        return resultado;
    }
  
        
    

//--- FUNÇÕES AUXILIARES -------------------------------------------------------
    /**
     * Esta função considera meses incompletos.
     *  Exemplo
     *      Data inicial.......:    01/10/2018;
     *      Data final.........:    18/10/2018;
     *      Resultado da função:    1 mês;
     * 
     *      Data inicial.......:    01/10/2018;
     *      Data final.........:    01/11/2018;
     *      Resultado da função:    2 mêses;
     * 
     * @param dataInicial
     * @param dataFinal
     * @return 
     */
    private int getDiferencaEmMesesEntreDatas(Calendar dataInicial, Calendar dataFinal) {
        int countMeses = 0;
        Calendar countData = Calendar.getInstance();
        countData.setTime(dataInicial.getTime());
        
        if (dataInicial != null && dataFinal != null && dataInicial.before(dataFinal)) {
            while (countData.get(Calendar.MONTH) != dataFinal.get(Calendar.MONTH) || countData.get(Calendar.YEAR) != dataFinal.get(Calendar.YEAR)) {
                countData.add(Calendar.MONTH, 1);
                countMeses++;
            }
            countMeses++; 
        }
        
        return countMeses;
    }
    
    private int getDiferencaEmMesesIntervaloDeDias(Calendar dataInicial, int intervaloEmDias) {
        
        Calendar dataFinal = Calendar.getInstance();
        dataFinal.setTime(dataInicial.getTime());
        dataFinal.add(Calendar.DAY_OF_MONTH, intervaloEmDias);
        
        return getDiferencaEmMesesEntreDatas(dataInicial, dataFinal);
    }
    
        private Double convertePercentualAnualParaMensal(Double rentabilidadeAnual) {

        // 1 + ia = (1 + ip)n onde ia é a taxa anual; ip é a taxa período e n é número de períodos
        // ip = (1 - raiz ((1 + ia),n)) * -1
        Double rentabilidadeMensal = null;
        Double numeroDePeriodos = 12.00;
        rentabilidadeAnual = rentabilidadeAnual / 100;

        rentabilidadeMensal = (1 - Math.pow(1 + rentabilidadeAnual, 1 / numeroDePeriodos)) * -1;

        return rentabilidadeMensal * 100;
    }
    
    /**
     * Recebe um valor e um percentual, por exemplo 17.5 (indicando 17,5 %) e retorna o valor
     * recebido deduzido do percentual.
     * 
     * @param valor:        Valor a ser deduzido do percentual.
     * @param percentual:   Valor a ser deduzido do percentual.
     * @return              Valor recebido deduzido do percentual.
     */
    private Double deduzPercentual(Double valor, Double percentual) {
        
        // deduzo apenas caso tenha havido rendimentos
        if (valor > 0.0) {
            return valor * (1 - percentual/100);
        } else {
            return valor;
        }
    }

}
