/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package br.com.flaviobarros.simuladordeinvestimentos.investimentos.indices.controller;

import br.com.flaviobarros.simuladordeinvestimentos.investimentos.indices.model.Indice;
import br.com.flaviobarros.util.GerenciadorArmazenamento;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 *
 * @author flavio
 */
public class GerenciaArmazenamentoIndices implements GerenciadorArmazenamento<Indice> {

    @Override  // OK!
    public boolean salva(Indice entidade) {
        boolean retorno = false;
        Long lastId;
        List<Indice> entidades;

        try {
            entidades = IndicesXML.load(); // arquivo de persistencia de dados não encontrado/não cadastrado
        } catch (FileNotFoundException ex) {
            entidades = new ArrayList<>();
        }

        // Obtendo id da ultima entidade cadastrada
        lastId = 0l;
        for (int i = 0; i < entidades.size(); i++) {
            if (lastId < entidades.get(i).getId()) {
                lastId = entidades.get(i).getId();
            }
        }

        // setando o id da entidade
        entidade.setId(lastId + 1);

        // persistindo entidade
        entidades.add(entidade);
        try {
            System.out.println("** ira persistir");
            IndicesXML.dump(entidades);
            retorno = true;
            System.out.println("** persistiu!");
        } catch (FileNotFoundException ex) {
            ex.printStackTrace();
        }
        
        return retorno;
    }

    @Override // OK!
    public boolean edita(Indice entidade) {
        
        List<Indice> entidades;
        boolean encontrado = false;
        boolean retorno = false;

        // carregando entidades
        try {
            entidades = IndicesXML.load(); // arquivo de persistencia de dados não encontrado/não cadastrado
        } catch (FileNotFoundException ex) {
            entidades = new ArrayList<>();
        }

        // buscando e alterando entidade
        for (int i = 0; i < entidades.size(); i++) {
            // buscando entidade
            if (Objects.equals(entidades.get(i).getId(), entidade.getId())) {
                entidades.set(i, entidade);
                encontrado = true;
                break;
            }
        }

        // salvando entidade editada
        if (encontrado) {
            try {
                System.out.println("** ira persistir entidade editada");
                IndicesXML.dump(entidades);
                retorno = true;
                System.out.println("** persistiu entidade editada!");
            } catch (FileNotFoundException ex) {
                ex.printStackTrace();
            }
        }
        
        return retorno;
        
    }

    @Override // OK!
    public boolean remove(Indice entidade) {
        
        List<Indice> entidades;
        boolean encontrado = false;
        boolean retorno = false;

        // carregando entidades já cadastrados
        try {
            entidades = IndicesXML.load(); // arquivo de persistencia de dados não encontrado/não cadastrado
        } catch (FileNotFoundException ex) {
            entidades = new ArrayList<>();
        }

        // buscando o entidade a ser removida
        for (int i = 0; i < entidades.size(); i++) {
            // buscando entidade
            if (Objects.equals(entidades.get(i).getId(), entidade.getId())) {
                entidades.remove(i);
                encontrado = true;
                break;
            }
        }

        // salvando entidades após exclusão
        if (encontrado) {
            try {
                System.out.println("** ira remover");
                IndicesXML.dump(entidades);
                retorno = true;
                System.out.println("** removeu!");
            } catch (FileNotFoundException ex) {
                ex.printStackTrace();
            }
        }

        return retorno;
        
    }

    @Override // OK!
    public Indice pesquisa(Long id) {
        
        List<Indice> entidades;

        try {
            entidades = IndicesXML.load(); // arquivo de persistencia de dados não encontrado/não cadastrado
        } catch (FileNotFoundException ex) {
            entidades = new ArrayList<>();
        }

        for (int i = 0; i < entidades.size(); i++) {
            if (Objects.equals(entidades.get(i).getId(), id)) {
                return entidades.get(i);
            }
        }

        return null;
    }

    @Override // OK!
    public List<Indice> getRegistros() {
        ArrayList<Indice> registros;

        try {
            //carrega os dados dos investimentos salvos (de acordo com o tipo recebido como parâmatro);
            registros = (ArrayList<Indice>) IndicesXML.load();
        } catch (FileNotFoundException ex) {
            //O arquivo com os dados dos investimentos salvos não existe, ou não pode ser carregado.

            //carrega uma lista de clientes vazia
            registros = new ArrayList<>();
        }

        return registros;
    }

    // OK!
    public boolean salvaOuAtualiza(Indice entidade) {
        if (entidade.getId() == 0L) {
            return this.salva(entidade);
        } else {
            return this.edita(entidade);
        }
    }

// OK!
//  pesquisaPorTipo(Indice.TIPO)
//      Esta função retorna um único tipo e não uma lista
//      O código deve garantir a inexistência de dois indices
//      de mesmo tipo cadastrados (UNIQUE).
    public Indice pesquisaPorTipo(Indice.TIPO tipo) {

        List<Indice> entidades;

        try {
            entidades = IndicesXML.load(); // arquivo de persistencia de dados não encontrado/não cadastrado
        } catch (FileNotFoundException ex) {
            entidades = new ArrayList<>();
        }

        for (int i = 0; i < entidades.size(); i++) {
            if (Objects.equals(entidades.get(i).getTipo(), tipo)) {
                return entidades.get(i);
            }
        }
        
        return null;
    }

}
