/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package br.com.flaviobarros.simuladordeinvestimentos.investimentos.model;

import java.io.Serializable;
import javax.persistence.Column;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.MappedSuperclass;

/**
 *
 * @author flavio
 */
@MappedSuperclass
public abstract class Investimento extends CenariosInvestimentos implements Serializable{
      
//--- DEFINIÇÃO DE TIPOS -------------------------------------------------------
    public enum TIPO{
        LCI("LCI"),
        LCA("LCA"),
        CDB("CDB"),
        RDB("RDB"),
        POUPANCA("Poupança"),
        TESOURO_SELIC("Tesouro Selic"),
        TESOURO_IPCA("Tesouro IPCA"),
        FUNDO_DE_INVESTIMENTO("Fundo de Investimento"),
        PREVIDENCIA("Previdência");        
        
        private final String descricao;

        private TIPO(String descricao) {
            this.descricao = descricao;
        }

        public String getDescricao() {
            return descricao;
        }
    }
    
    
    
//--- ATRIBUTOS ----------------------------------------------------------------
    @Id
    @GeneratedValue
    private Long id;
    
    //Nome identificador do investimento
    @Column(nullable = false, length = 60)
    private String nome;
       
    //Tipo do investimento
    @Column(nullable = false)
    @Enumerated(EnumType.STRING)
    private final TIPO tipo;
    
    
    
//--- CONSTRUTOR ---------------------------------------------------------------
    public Investimento(String nome, TIPO tipo) {
        super();
        this.nome = nome;
        this.tipo = tipo;
        this.id = 0l;
    }

    
    
//--- MÉTODOS ------------------------------------------------------------------
    public String getDescricao() {
        return "\n   Nome: "+this.getNome()+"\n\n"+
               "   Tipo: "+this.getTipo().getDescricao()+"\n\n";
    }
    
    //método abstrato a ser utilizado para exibição no relatório
    public String getDescricaoCompacta(){
        return "Nome: "+this.getNome()+
               "; Tipo: "+this.getTipo().getDescricao()+".";
    }
    
    
// FUNÇÕES PARA IDENTIFICAÇÃO DO INVESTIMENTO ----------------------------------
    public boolean isLCI() {
        return this.getTipo() == Investimento.TIPO.LCI;
    }
    
    public boolean isLCA() {
        return this.getTipo() == Investimento.TIPO.LCA;
    }
    
    public boolean isCDB() {
        return this.getTipo() == Investimento.TIPO.CDB;
    }
    
    public boolean isRDB() {
        return this.getTipo() == Investimento.TIPO.RDB;
    }
    
    public boolean isPoupanca() {
        return this.getTipo() == Investimento.TIPO.POUPANCA;
    }
    
    public boolean isTesouroSelic() {
        return this.getTipo() == Investimento.TIPO.TESOURO_SELIC;
    }
    
    public boolean isTesouroIPCA() {
        return this.getTipo() == Investimento.TIPO.TESOURO_IPCA;
    }
    
    public boolean isFundoDeInvestimento() {
        return this.getTipo() == Investimento.TIPO.FUNDO_DE_INVESTIMENTO;
    }
    
    public boolean isPrevidencia() {
        return this.getTipo() == Investimento.TIPO.PREVIDENCIA;
    }
    
    
    
//--- GETTERS AND SETTERS ------------------------------------------------------
    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }
    
    public TIPO getTipo() {
        return tipo;
    }  
    
    public String getNome() {
        return nome;
    }

    public void setNome(String nome) {
        this.nome = nome;
    }
    
}
