/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package br.com.flaviobarros.simuladordeinvestimentos.investimentos.model;

import java.io.Serializable;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.Table;

/**
 *
 * @author flavio
 */
@Entity
@Table (name = "investimento_previdencia")
public class InvestimentoPrevidencia extends Investimento implements Serializable{
    
//--- CONSTANTES ---------------------------------------------------------------
    public static String OBSERVACAO_QUANTO_A_PLANOS_PROGRESSIVOS = 
            "Previdências com o regime de tributação Progressiva Compensável estão sujeitas" +
            " ao ajuste do percentual de imposto de renda pago na declaração anual de ajuste" +
            " imposto de renda do ano subsequente ao resgate.";
    public static String OBSERVACAO_QUANTO_A_PLANOS_PGBL =
            "Fique atento: nos planos de previdência PGBL só estão sujeito a benefício fiscal "+
            " aportes cuja soma durante o ano não ultrapasse o limite de 12 % da renda bruta anual do investidor.";
    
    
    
//--- TIPOS --------------------------------------------------------------------
    public enum TIPO_PREVIDENCIA{
        PGBL("PGBL"),// PGBL (Plano Gerador de Benefício Livre)
        VGBL("VGBL");// VGBL (Vida Gerador de Benefício Livre)

        private final String descricao;

        private TIPO_PREVIDENCIA(String descricao) {
            this.descricao = descricao;
        }

        public String getDescricao() {
            return descricao;
        }
    }
        
    public enum TRIBUTACAO_PREVIDENCIA{
        PROGRESSIVA("Progressiva Compensavel"),
        REGRESSIVA("Regressiva Definitiva");

        private final String descricao;

        private TRIBUTACAO_PREVIDENCIA(String descricao) {
            this.descricao = descricao;
        }

        public String getDescricao() {
            return descricao;
        }
    }
    
    
    
//--- ATRIBUTOS ----------------------------------------------------------------
    //Tipo do plano de previdência (PGBL ou VGBL)
    @Column(nullable = false, name = "tipo_previdencia")
    @Enumerated(EnumType.STRING)
    private TIPO_PREVIDENCIA tipoPrevidencia;
    
    //Regime de tributação do plano de previdência (PGBL ou VGBL)
    @Column(nullable = false, name = "tributacao_previdencia")
    @Enumerated(EnumType.STRING)
    private TRIBUTACAO_PREVIDENCIA tributacaoPrevidencia;

    @Column(nullable = false, name = "taxa_carregamento_antecipada")
    private Double taxaDeCarregamentoAntecipada;
    
    @Column(nullable = false, name = "taxa_carregamento_postecipada")
    private Double taxaDeCarregamentoPostecipada;

    @Column(nullable = false, name = "carencia_resgate")
    private Integer carenciaResgate;


    
//--- CONSTRUTORES -------------------------------------------------------------        
    public InvestimentoPrevidencia() {
        super("", Investimento.TIPO.PREVIDENCIA);
        this.tipoPrevidencia = null;
        this.tributacaoPrevidencia = null;
        this.taxaDeCarregamentoAntecipada = 0.0;
        this.taxaDeCarregamentoPostecipada = 0.0;
        this.carenciaResgate = 0;
    }
    

    
//--- OUTROS MÉTODOS -----------------------------------------------------------

    @Override
    public String getDescricao() {
        String descricao;
        
        descricao = super.getDescricao();
        descricao = descricao + "   Tipo de plano: "+this.getTipoPrevidencia().getDescricao()+"\n\n";
        descricao = descricao + "   Tipo de tributação: "+this.getTributacaoPrevidencia().getDescricao()+"\n\n";
        descricao = descricao + "   Carência: "+this.getCarenciaResgate()+" dias"+"\n\n";
        descricao = descricao + "   Taxa de carregamento:\n";
        descricao = descricao + "      - antecipada: "+this.getTaxaDeCarregamentoAntecipada()+" %\n";
        descricao = descricao + "      - postecipada: "+this.getTaxaDeCarregamentoPostecipada()+" %\n\n";
        
        return  descricao;
    }
    
    @Override
    public String getDescricaoCompacta() {
        
        String descricao = "Nome: "+this.getNome()+"\n";
        descricao +=  "   Tipo: "+this.getTipo().getDescricao();
        descricao +=  ";  Tipo de plano: "+this.getTipoPrevidencia().getDescricao();
        descricao +=  ";  Tipo de tributação: "+this.getTributacaoPrevidencia().getDescricao();
        descricao +=  ";  Carência: "+this.getCarenciaResgate()+" dias";
        descricao +=  ";  Taxa de carregamento antecipada: "+this.getTaxaDeCarregamentoAntecipada()+" %";
        descricao +=  ";  Taxa de carregamento postecipada: "+this.getTaxaDeCarregamentoPostecipada()+" %.";
        
        return descricao;
    }
    
    public boolean isPGBL() {
        return this.getTipoPrevidencia() == TIPO_PREVIDENCIA.PGBL;
    }
    
    public boolean isVGBL() {
        return this.getTipoPrevidencia() == TIPO_PREVIDENCIA.VGBL;
    }
    
    public boolean isProgressiva() {
        return this.getTributacaoPrevidencia() == TRIBUTACAO_PREVIDENCIA.PROGRESSIVA;
    }
    
    public boolean isRegressiva() {
        return this.getTributacaoPrevidencia() == TRIBUTACAO_PREVIDENCIA.REGRESSIVA;
    }
    
    
    
//--- GETTERS AND SETTERS PADRÕES ----------------------------------------------

    public TIPO_PREVIDENCIA getTipoPrevidencia() {
        return tipoPrevidencia;
    }

    public void setTipoPrevidencia(TIPO_PREVIDENCIA tipo_previdencia) {
        this.tipoPrevidencia = tipo_previdencia;
    }

    public TRIBUTACAO_PREVIDENCIA getTributacaoPrevidencia() {
        return tributacaoPrevidencia;
    }

    public void setTributacaoPrevidencia(TRIBUTACAO_PREVIDENCIA tributacao_previdencia) {
        this.tributacaoPrevidencia = tributacao_previdencia;
    }

    public Double getTaxaDeCarregamentoAntecipada() {
        return taxaDeCarregamentoAntecipada;
    }

    public void setTaxaDeCarregamentoAntecipada(Double taxaDeCarregamentoAntecipada) {
        this.taxaDeCarregamentoAntecipada = taxaDeCarregamentoAntecipada;
    }

    public Double getTaxaDeCarregamentoPostecipada() {
        return taxaDeCarregamentoPostecipada;
    }

    public void setTaxaDeCarregamentoPostecipada(Double taxaDeCarregamentoPostecipada) {
        this.taxaDeCarregamentoPostecipada = taxaDeCarregamentoPostecipada;
    }

    public Integer getCarenciaResgate() {
        return carenciaResgate;
    }

    public void setCarenciaResgate(Integer carenciaResgate) {
        this.carenciaResgate = carenciaResgate;
    }
    
}
