/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package br.com.flaviobarros.simuladordeinvestimentos.investimentos.model;

import br.com.flaviobarros.util.FuncoesDeDatas;
import java.io.Serializable;
import java.util.Calendar;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Table;
import javax.persistence.Temporal;

/**
 *
 * @author flavio
 */
@Entity
@Table (name = "investimento_tesouro_ipca")
public class InvestimentoTesouroIPCA extends Investimento implements Serializable{
    
//--- CONSTANTES ---------------------------------------------------------------
public static String OBSERVACAO_QUANTO_A_MARCACAO_A_MERCADO =
        "Investimentos no Tesouro Direto tem sua rentabilidade para resgates antes"+
         " do prazo de vencimento do título variando por meio de marcação a"+
         " mercado. Os valores mostrado na simulação são portanto proporcionais"+
         " a rentabilidade do título contratada e ao tempo de investimento simulado,"+
         " para fins de comparação de rentabilidade com a rentabilidade de outros"+
         " investimentos. Apenas para títulos resgatados na data de investimentos a"+
         " rentabilidade utilizada será a contratada.";
    
         
         
//--- ATRIBUTOS ----------------------------------------------------------------
    //Data de vencimento da aplicação
    @Column(nullable = false)
    @Temporal(javax.persistence.TemporalType.DATE)
    private Calendar vencimento;
    
    //Rentabilidade adicional ao IPCA contratada
    @Column(nullable = false, name = "rentabilidade_adicional_contratada")
    private Double rentabilidadeAdicionalContratada;

    
    
//--- CONSTRUTORES -------------------------------------------------------------    
    public InvestimentoTesouroIPCA() {
        super("", Investimento.TIPO.TESOURO_IPCA);
        this.rentabilidadeAdicionalContratada = 0.0;
        this.vencimento = null;
    }
    
    
    
//--- MÉTODOS ------------------------------------------------------------------
    @Override
    public String getDescricao() {
        String descricao;
        
        descricao = super.getDescricao();
        descricao = descricao + "   Rentabilidade adicional: "+this.getRentabilidadeAdicionalContratada()+" % a.a.\n\n";
        descricao = descricao + "   Vencimento: "+FuncoesDeDatas.getDataDDMMAAAA(this.getVencimento())+"\n\n";
        
        return  descricao;
    }
    
    @Override
    public String getDescricaoCompacta() {
        
        String descricao = "Nome: "+this.getNome()+"\n"+
                            "   Tipo: "+this.getTipo().getDescricao()+
                            ";  Rentabilidade adicional: "+this.getRentabilidadeAdicionalContratada()+" % a.a."+
                            ";  Vencimento: "+ FuncoesDeDatas.getDataDDMMAAAA(this.getVencimento())+".";
        
        return descricao;
    }
    
    
    
//--- GETTERS AND SETTERS ------------------------------------------------------
    
    public Calendar getVencimento() {
        return vencimento;
    }

    public void setVencimento(Calendar vencimento) {
        this.vencimento = vencimento;
    }

    public Double getRentabilidadeAdicionalContratada() {
        return rentabilidadeAdicionalContratada;
    }

    public void setRentabilidadeAdicionalContratada(Double rentabilidadeAdicionalContratada) {
        this.rentabilidadeAdicionalContratada = rentabilidadeAdicionalContratada;
    }
    
}
