/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package br.com.flaviobarros.simuladordeinvestimentos.investimentos.model;

import br.com.flaviobarros.util.FuncoesDeDatas;
import java.io.Serializable;
import java.util.Calendar;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Table;
import javax.persistence.Temporal;

/**
 *
 * @author flavio
 */
@Entity
@Table (name = "investimento_tesouro_selic")
public class InvestimentoTesouroSelic extends Investimento implements Serializable{
    
    public final static String OBSERVACAO_QUANTO_A_MARCACAO_A_MERCADO =
        "Investimentos no Tesouro Direto tem sua rentabilidade para resgates antes"+
         " do prazo de vencimento do título variando por meio de marcação a"+
         " mercado. Os valores mostrado na simulação são portanto proporcionais"+
         " a rentabilidade do título contratada e ao tempo de investimento simulado,"+
         " para fins de comparação com a rentabilidade de outros"+
         " investimentos. Apenas para títulos resgatados na data de vencimento acordada na compra a"+
         " rentabilidade utilizada será a contratada.";
    
//--- ATRIBUTOS ----------------------------------------------------------------
    //Data de vencimento da aplicação
    @Column(nullable = false)
    @Temporal(javax.persistence.TemporalType.DATE)
    private Calendar vencimento;
    
    @Column(nullable = false, name = "rendimento_adicional")
    private Double rendimentoAdicional;

    
    
//--- CONSTRUTORES -------------------------------------------------------------
    public InvestimentoTesouroSelic() {
        super("", Investimento.TIPO.TESOURO_SELIC);
        this.vencimento= null;
        this.rendimentoAdicional = 0.0;
    }


    
//--- MÉTODOS ------------------------------------------------------------------
    @Override
    public String getDescricao() {
        String descricao;
        
        descricao = super.getDescricao();
        descricao += "   Vencimento: "+FuncoesDeDatas.getDataDDMMAAAA(this.getVencimento())+"\n\n";
        descricao += "   Rendimento adicional: "+this.getRendimentoAdicional()+" % a.a.";
        
        return  descricao;
    }
    
    @Override
    public String getDescricaoCompacta() {
        
        String descricao = "Nome: "+this.getNome()+"\n"+
                            "   Tipo: "+this.getTipo().getDescricao()+
                            ";  Vencimento: "+ FuncoesDeDatas.getDataDDMMAAAA(this.getVencimento())+
                            ";  Rendimento adicional: "+this.getRendimentoAdicional()+" % a.a.";
        
        return descricao;
    }
    
    
    
//--- GETTERS AND SETTERS ------------------------------------------------------
    public Calendar getVencimento() {
        return vencimento;
    }

    public void setVencimento(Calendar vencimento) {
        this.vencimento = vencimento;
    }

    public Double getRendimentoAdicional() {
        return rendimentoAdicional;
    }

    public void setRendimentoAdicional(Double rendimentoAdicional) {
        this.rendimentoAdicional = rendimentoAdicional;
    }
    
    
    
}
