/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package br.com.flaviobarros.simuladordeinvestimentos.investimentos.model;

import java.io.Serializable;
import javax.persistence.Column;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.MappedSuperclass;

/**
 *
 * @author flavio
 * 
 * Classe com atributos que são aplicaveis aos investimentos em, CDB, RDB, LCI e LCA
 * 
 */
@MappedSuperclass
public abstract class InvestimentosComunsSemelhantes extends Investimento implements Serializable{
    
//--- DEFINIÇÃO DE TIPOS -------------------------------------------------------
    public enum TIPO_DE_RENTABILIDADE{
        PRE_FIXADO("Pré Fixada"),
        POS_FIXADO("Pós Fixada");     
        
        private final String descricao;

        private TIPO_DE_RENTABILIDADE(String descricao) {
            this.descricao = descricao;
        }
        
        public String getDescricao() {
            return descricao;
        }
    }
    
    
    
//--- ATRIBUTOS ----------------------------------------------------------------
    //pré ou pós fixada
    @Column(nullable = false, name = "tipo_de_rentabilidade")
    @Enumerated(EnumType.STRING)
    private TIPO_DE_RENTABILIDADE tipoDeRentabilidade;
    
    //percentual do CDI
    @Column(nullable = true, name = "percentual_cdi_contratado")
    private Double percentualCDIContratado;
    
    //prazo do investimento em dias
    @Column(nullable = false, name = "prazo_do_investimento")
    private Integer prazoDoInvestimento;

    
    
//--- CONSTRUTORES -------------------------------------------------------------
    public InvestimentosComunsSemelhantes(String nome, TIPO tipo) {
        super(nome, tipo);
        this.tipoDeRentabilidade = null;
        this.percentualCDIContratado = 0.0;
        this.prazoDoInvestimento = 0;
        
    }
  

    
//--- MÉTODOS ------------------------------------------------------------------
    @Override
    public String getDescricao() {
        
        String descricao;
        
        descricao = super.getDescricao();
        descricao = descricao + "   Tipo de rentabilidade: "+getTipoDeRentabilidade().getDescricao()+"\n\n";
        
        if(getTipoDeRentabilidade() == InvestimentosComunsSemelhantes.TIPO_DE_RENTABILIDADE.POS_FIXADO){
            descricao = descricao + "   Rentabilidade: " + getPercentualCDIContratado();
            descricao = descricao + " % do CDI.\n\n";
        }
        
        descricao = descricao + "   Prazo: " + this.getPrazoDoInvestimento() + " dias\n\n";
        
        return  descricao;
    }
    
    @Override
    public String getDescricaoCompacta() {
        
        String descricao = "Nome: "+this.getNome()+"\n"+
                            "   Tipo: "+this.getTipo().getDescricao()+
                            ";  Tipo de rentabilidade: "+this.getTipoDeRentabilidade().getDescricao();
                            
        if(getTipoDeRentabilidade() == InvestimentosComunsSemelhantes.TIPO_DE_RENTABILIDADE.POS_FIXADO){
            descricao += ";  Rentabilidade contratada: " + getPercentualCDIContratado();
            descricao += " % do CDI";
        }
        
        descricao += ";  Prazo: "+ this.getPrazoDoInvestimento() + " dias.";
        
        
        return descricao;
    }
    
//- Funções para identificar se o investimento é pré ou pós fixado -----------
    public boolean isPrefixado() {
        return this.getTipoDeRentabilidade() == InvestimentosComunsSemelhantes.TIPO_DE_RENTABILIDADE.PRE_FIXADO;
    }
    
    public boolean isPosFixado() {
        return this.getTipoDeRentabilidade() == InvestimentosComunsSemelhantes.TIPO_DE_RENTABILIDADE.POS_FIXADO;
    }
    
    
    
//--- GETTERS AND SETTERS ------------------------------------------------------
    public TIPO_DE_RENTABILIDADE getTipoDeRentabilidade() {
        return tipoDeRentabilidade;
    }

    public void setTipoDeRentabilidade(TIPO_DE_RENTABILIDADE tipoDeRentabilidade) {
        this.tipoDeRentabilidade = tipoDeRentabilidade;
    }

    public Double getPercentualCDIContratado() {
        return percentualCDIContratado;
    }

    public void setPercentualCDIContratado(Double rentabilidadeContratada) {
        this.percentualCDIContratado = rentabilidadeContratada;
    }

    public Integer getPrazoDoInvestimento() {
        return prazoDoInvestimento;
    }

    public void setPrazoDoInvestimento(Integer prazoDoInvestimento) {
        this.prazoDoInvestimento = prazoDoInvestimento;
    }
    
}
