/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package br.com.flaviobarros.simuladordeinvestimentos.relatorio.controller;

import br.com.flaviobarros.simuladordeinvestimentos.investimentos.controller.CalculaRentabilidade;
import br.com.flaviobarros.simuladordeinvestimentos.investimentos.controller.GerenciaArmazenamentoInvestimentos;
import br.com.flaviobarros.simuladordeinvestimentos.investimentos.indices.controller.GerenciaArmazenamentoIndices;
import br.com.flaviobarros.simuladordeinvestimentos.investimentos.indices.model.Indice;
import br.com.flaviobarros.simuladordeinvestimentos.investimentos.model.Investimento;
import br.com.flaviobarros.simuladordeinvestimentos.investimentos.model.InvestimentoPrevidencia;
import br.com.flaviobarros.simuladordeinvestimentos.investimentos.model.InvestimentoTesouroIPCA;
import br.com.flaviobarros.simuladordeinvestimentos.relatorio.model.ModeloItemGrafico;
import br.com.flaviobarros.simuladordeinvestimentos.relatorio.model.ParametroRelatorioTabelaIndice;
import br.com.flaviobarros.simuladordeinvestimentos.relatorio.model.ParametroRelatorioTabelaRentabilidade;
import br.com.flaviobarros.simuladordeinvestimentos.relatorio.model.ParametroStringRelatorio;
import br.com.flaviobarros.simuladordeinvestimentos.relatorio.model.RentabilidadeInvestimento;
import br.com.flaviobarros.simuladordeinvestimentos.simulacao.model.DadosSimulacao;
import br.com.flaviobarros.util.FuncoesDeDatas;
import br.com.flaviobarros.util.FuncoesDeMoedas;
import java.awt.image.BufferedImage;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.sf.jasperreports.engine.JREmptyDataSource;
import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.data.JRBeanCollectionDataSource;
import net.sf.jasperreports.engine.util.JRLoader;

/**
 *
 * @author flavio
 */
public class GerenciaRelatorio {

//--- TIPOS --------------------------------------------------------------------------------------------------//
    public enum ESCALA_CASAS_DECIMAIS_GRAFICO {
        EM_QUADRILHOES,
        EM_TRILHOES,
        EM_BILHOES,
        EM_MILHOES,
        NORMAL
    }

//---- CONSTANTES --------------------------------------------------------------------------------------------//
    final String NOME_RELATORIO = "relatorio_simulacao_investimentos.jasper";
    final String CAMINHO_RELATORIO = "/br/com/flaviobarros/simuladordeinvestimentos/relatorio/jasper/";

//---- ATRIBUTOS ----------------------------------------------------------------------------------------------//    
//  contem os investimentos
    private List<Investimento> investimentos;

//  contem os índices
    private Indice indiceCDI;
    private Indice indiceSelic;
    private Indice indiceIPCA;
    private Indice indiceTR;

//  conterá as dados calculados da rentabilidade dos investimentos
    private List<RentabilidadeInvestimento> rentabilidadeDosInvestimentos;

//  conterá os dados da simulação 
    private final DadosSimulacao dadosSimulacao;

// conterá o relatório assim que o mesmo for gerado
    private JasperPrint jpRelatorio;

//---- CONSTRUTOR ---------------------------------------------------------------------------------------------//    
    /**
     * Construtor da classe
     *
     * @param dados: Classe contendo os dados da simulação a ser realizada
     */
    public GerenciaRelatorio(DadosSimulacao dados) {
        this.dadosSimulacao = dados;

        inicializaClasse();
    }

//---- ESTA SERA A ÚNICA FUNÇÂO PULBLICA DESTA CLASSE ----------------------------------------------------------//
    public JasperPrint getRelatorio() throws JRException, Exception {

        if (this.jpRelatorio == null) {
            this.geraRelatorio();
        }

        return jpRelatorio;
    }

//---- FUNÇÕES PRIVADAS ----------------------------------------------------------------------------------------//    
//---- Funções de carregamento e inicializações da Classe
    /**
     * ESTA FUNÇÂO DEVE SER A PRIMEIRA A SER EXECULTADA NA CLASSE.
     *
     * Realiza as inicializações básicas da classe.
     */
    private void inicializaClasse() {
        iniciaEstruturas();
        loadFirstInvestimentos();
        loadSecondIndices();
        loadThirdRentabilidades();
    }

    private void iniciaEstruturas() {
        this.investimentos = new ArrayList<>();
        this.rentabilidadeDosInvestimentos = new ArrayList<>();

        this.jpRelatorio = null;

        this.indiceCDI = null;
        this.indiceIPCA = null;
        this.indiceSelic = null;
        this.indiceTR = null;
    }

    private void loadFirstInvestimentos() {
        // obtendo os dados dos investimentos do banco de dados
        GerenciaArmazenamentoInvestimentos gerenciadorInvestimentos = new GerenciaArmazenamentoInvestimentos();
        this.investimentos = gerenciadorInvestimentos.getRegistrosDeTodosOsInvestimentos();
    }

    private void loadSecondIndices() {
        // obtendo os dados dos indices do banco de dados
        GerenciaArmazenamentoIndices gerenciaArmazenamentoIndices = new GerenciaArmazenamentoIndices();
        this.indiceCDI = gerenciaArmazenamentoIndices.pesquisaPorTipo(Indice.TIPO.CDI);
        this.indiceSelic = gerenciaArmazenamentoIndices.pesquisaPorTipo(Indice.TIPO.SELIC);
        this.indiceIPCA = gerenciaArmazenamentoIndices.pesquisaPorTipo(Indice.TIPO.IPCA);
        this.indiceTR = gerenciaArmazenamentoIndices.pesquisaPorTipo(Indice.TIPO.TR);
    }

    private void loadThirdRentabilidades() {
        // criando o objeto responsavel pelo calculo das rentabilidades
        CalculaRentabilidade calculaRentabilidade = new CalculaRentabilidade(this.indiceCDI, this.indiceSelic, this.indiceIPCA, this.indiceTR);

        this.rentabilidadeDosInvestimentos = new ArrayList<>();

        for (int i = 0; i < this.investimentos.size(); i++) {
            RentabilidadeInvestimento rentabilidade = calculaRentabilidade.calcula(this.investimentos.get(i), this.dadosSimulacao.getDuracaoInvestimento(), this.dadosSimulacao.getIntervaloRelatorioEmMeses(), this.dadosSimulacao.getValorInvestimento(), this.dadosSimulacao.getMesAnoAplicacao());
            this.rentabilidadeDosInvestimentos.add(rentabilidade);
        }
    }

    private int ajustaQuantidadeIntervalosGrafico24unidades(int quantidadeDeIntervalos) {

        int valorIntervaloAjustado = quantidadeDeIntervalos / 24;

        valorIntervaloAjustado += 1;

        return valorIntervaloAjustado;
    }

    /**
     * ESTA FUNÇÂO DEVE SER EXECULTADA APÓS AS FUNÇÕES DE INICIALIZAÇÔES,
     * CARREGAMENTO, E CALCULOS INICIAIS
     */
    private List<ModeloItemGrafico> getDadosGrafico(int numeroCenario) {

        // --- obtendo o identificador do cenárioa a ser utilizado
        RentabilidadeInvestimento.IDENTIFICADOR_CENARIO cenario;

        switch (numeroCenario) {
            case 1:
                cenario = RentabilidadeInvestimento.IDENTIFICADOR_CENARIO.A;
                break;
            case 2:
                cenario = RentabilidadeInvestimento.IDENTIFICADOR_CENARIO.B;
                break;
            case 3:
                cenario = RentabilidadeInvestimento.IDENTIFICADOR_CENARIO.C;
                break;
            case 4:
                cenario = RentabilidadeInvestimento.IDENTIFICADOR_CENARIO.D;
                break;
            case 5:
                cenario = RentabilidadeInvestimento.IDENTIFICADOR_CENARIO.E;
                break;
            case 6:
                cenario = RentabilidadeInvestimento.IDENTIFICADOR_CENARIO.F;
                break;
            default:
                throw new IllegalArgumentException("Número do cenário recebido como parâmetro inválido!");
        }

        // --- iniciando a obtenção dos dados do gráfico
        List<ModeloItemGrafico> listaItensGraficos = new ArrayList<>();

        int quantidadeDeIntervalos = this.dadosSimulacao.getDuracaoInvestimento() / this.dadosSimulacao.getIntervaloRelatorioEmMeses();
        int quantidadeDeInvestimentos = this.investimentos.size();

        //System.out.println("\n*** Quantidade intervalos: " + quantidadeDeIntervalos + " Quantidade investimentos: " + quantidadeDeInvestimentos + "\n");
        // esta função retorna a quantidade de intervalos que necessitam ser pulados de cada vez para que
        // o eixo x do gráfico não exiba mais que 24 intervalos
        int valorIntervaloAjustado = ajustaQuantidadeIntervalosGrafico24unidades(quantidadeDeIntervalos);

        int mesIntervalo = valorIntervaloAjustado * this.dadosSimulacao.getIntervaloRelatorioEmMeses();

        for (int i = 0; i < quantidadeDeIntervalos; i += valorIntervaloAjustado) {

            for (int contInvestimento = 0; contInvestimento < quantidadeDeInvestimentos; contInvestimento++) {

                String nomeInvestimento = this.investimentos.get(contInvestimento).getNome();
                Double montante = this.rentabilidadeDosInvestimentos.get(contInvestimento).getRentabilidadeCenarioEspecifico(cenario, i);

                ModeloItemGrafico item = new ModeloItemGrafico(mesIntervalo, nomeInvestimento, montante);
                listaItensGraficos.add(item);
            }

            mesIntervalo += valorIntervaloAjustado * this.dadosSimulacao.getIntervaloRelatorioEmMeses();
        }

        return listaItensGraficos;
    }

    /**
     * Retorna se as casas decimais do grafico devem ser exibidadas em unidades,
     * em milhões ou em bilhões de acordo com o tamanho da rentabilidade dos
     * investimentos calculadas.
     *
     * @return ESCALA_CASAS_DECIMAIS_GRAFICO escala a ser utilizada na exibição
     * dos dados de rentabilidade no gráfico
     */
    private ESCALA_CASAS_DECIMAIS_GRAFICO getEscalaCasasDecimaisGrafico() {

        final Double valorLimiteMilhoes = 100000000.00 - 1;
        final Double valorLimiteBilhoes = 1000000000000.00 - 1;
        final Double valorLimiteTrilhoes = 1000000000000000.00 - 1;
        final Double valorLimiteQuadrilhoes = 1000000000000000000.00 - 1;

        ESCALA_CASAS_DECIMAIS_GRAFICO retorno = ESCALA_CASAS_DECIMAIS_GRAFICO.NORMAL;

        if (this.dadosSimulacao.getValorInvestimento() > 100000 || this.dadosSimulacao.getDuracaoInvestimento() > 120) {

            for (int i = 0; i < this.rentabilidadeDosInvestimentos.size(); i++) {

                int quantidadeIntervalosSimulacao = this.dadosSimulacao.getDuracaoInvestimento() / this.dadosSimulacao.getIntervaloRelatorioEmMeses();

                for (int j = 0; j < quantidadeIntervalosSimulacao; j++) {

                    if (this.rentabilidadeDosInvestimentos.get(i).getRentabilidadeCenario1(j) != null) {
                        if (this.rentabilidadeDosInvestimentos.get(i).getRentabilidadeCenario1(j) > valorLimiteQuadrilhoes
                                || this.rentabilidadeDosInvestimentos.get(i).getRentabilidadeCenario2(j) > valorLimiteQuadrilhoes
                                || this.rentabilidadeDosInvestimentos.get(i).getRentabilidadeCenario3(j) > valorLimiteQuadrilhoes
                                || this.rentabilidadeDosInvestimentos.get(i).getRentabilidadeCenario4(j) > valorLimiteQuadrilhoes
                                || this.rentabilidadeDosInvestimentos.get(i).getRentabilidadeCenario5(j) > valorLimiteQuadrilhoes
                                || this.rentabilidadeDosInvestimentos.get(i).getRentabilidadeCenario6(j) > valorLimiteQuadrilhoes) {
                            return ESCALA_CASAS_DECIMAIS_GRAFICO.EM_QUADRILHOES;
                        }

                        if (this.rentabilidadeDosInvestimentos.get(i).getRentabilidadeCenario1(j) > valorLimiteTrilhoes
                                || this.rentabilidadeDosInvestimentos.get(i).getRentabilidadeCenario2(j) > valorLimiteTrilhoes
                                || this.rentabilidadeDosInvestimentos.get(i).getRentabilidadeCenario3(j) > valorLimiteTrilhoes
                                || this.rentabilidadeDosInvestimentos.get(i).getRentabilidadeCenario4(j) > valorLimiteTrilhoes
                                || this.rentabilidadeDosInvestimentos.get(i).getRentabilidadeCenario5(j) > valorLimiteTrilhoes
                                || this.rentabilidadeDosInvestimentos.get(i).getRentabilidadeCenario6(j) > valorLimiteTrilhoes) {
                            retorno = ESCALA_CASAS_DECIMAIS_GRAFICO.EM_TRILHOES;
                        }

                        if (this.rentabilidadeDosInvestimentos.get(i).getRentabilidadeCenario1(j) > valorLimiteBilhoes
                                || this.rentabilidadeDosInvestimentos.get(i).getRentabilidadeCenario2(j) > valorLimiteBilhoes
                                || this.rentabilidadeDosInvestimentos.get(i).getRentabilidadeCenario3(j) > valorLimiteBilhoes
                                || this.rentabilidadeDosInvestimentos.get(i).getRentabilidadeCenario4(j) > valorLimiteBilhoes
                                || this.rentabilidadeDosInvestimentos.get(i).getRentabilidadeCenario5(j) > valorLimiteBilhoes
                                || this.rentabilidadeDosInvestimentos.get(i).getRentabilidadeCenario6(j) > valorLimiteBilhoes) {
                            retorno = ESCALA_CASAS_DECIMAIS_GRAFICO.EM_BILHOES;
                        }

                        if (this.rentabilidadeDosInvestimentos.get(i).getRentabilidadeCenario1(j) > valorLimiteMilhoes
                                || this.rentabilidadeDosInvestimentos.get(i).getRentabilidadeCenario2(j) > valorLimiteMilhoes
                                || this.rentabilidadeDosInvestimentos.get(i).getRentabilidadeCenario3(j) > valorLimiteMilhoes
                                || this.rentabilidadeDosInvestimentos.get(i).getRentabilidadeCenario4(j) > valorLimiteMilhoes
                                || this.rentabilidadeDosInvestimentos.get(i).getRentabilidadeCenario5(j) > valorLimiteMilhoes
                                || this.rentabilidadeDosInvestimentos.get(i).getRentabilidadeCenario6(j) > valorLimiteMilhoes) {
                            retorno = ESCALA_CASAS_DECIMAIS_GRAFICO.EM_MILHOES;
                        }
                    }
                }
            }
        }

        return retorno;
    }

    /**
     * ESTA FUNÇÂO DEVE SER EXECULTADA APÓS AS FUNÇÕES DE INICIALIZAÇÔES,
     * CARREGAMENTO, E CALCULOS INICIAIS
     */
    private BufferedImage getGraficoPorCenario(Integer numeroCenario) throws Exception {

        if (numeroCenario > 6 || numeroCenario < 1) {
            throw new IllegalArgumentException("Número do cenário recebido como parâmetro inválido!");
        }

        String titulo = "Rentabilidade investimentos cenário " + numeroCenario.toString();

        return GeradorDeGrafico.gerarGraficoRentabilidade(titulo, (ArrayList) getDadosGrafico(numeroCenario), getEscalaCasasDecimaisGrafico());
    }

    private void geraRelatorio() throws JRException, Exception {
 
        // obtem os dados dos cenários e rentabilidade investimentos para exibição no relatório
        List<ParametroRelatorioTabelaIndice> listaRelatorioCenarios = this.getDadosRelatorioTabelaCenarios();
        List<ParametroRelatorioTabelaRentabilidade> dadosRelatorioTabelaRentabilidades = this.getDadosRelatorioTabelaRentabilidades();

        // carregando arquivo do relatório Jasper
        URL url = this.getClass().getResource(CAMINHO_RELATORIO + NOME_RELATORIO);
        JasperReport report = (JasperReport) JRLoader.loadObject(url);

        // criando o parâmetro listaIndices
        JRBeanCollectionDataSource JRBeanCenarios = new JRBeanCollectionDataSource(listaRelatorioCenarios);
        JRBeanCollectionDataSource JRBeanRentabilidadeInvestimentos = new JRBeanCollectionDataSource(dadosRelatorioTabelaRentabilidades);
        JRBeanCollectionDataSource JRBeanDescricaoInvestimentos = new JRBeanCollectionDataSource(this.getDescricaoInvestimentos());
        JRBeanCollectionDataSource JRBeanMensagensRodape = new JRBeanCollectionDataSource(this.getMensagensDeRodape());

        Map<String, Object> parametros = new HashMap<String, Object>();
        parametros.put("LISTA_CENARIOS", JRBeanCenarios);
        parametros.put("LISTA_RENTABILIDADE_INVESTIMENTOS", JRBeanRentabilidadeInvestimentos);
        parametros.put("LISTA_DESCRICAO_INVESTIMENTOS", JRBeanDescricaoInvestimentos);
        parametros.put("LISTA_MENSAGENS_RODAPE", JRBeanMensagensRodape);

        // Gráficos do cenário
        if (this.dadosSimulacao.isSimularCenario1() && !investimentos.isEmpty()) {
            parametros.put("GRAFICO_CENARIO_1", getGraficoPorCenario(1));
        } else {
            parametros.put("GRAFICO_CENARIO_1", null);
        }

        // Gráficos do cenário
        if (this.dadosSimulacao.isSimularCenario2() && !investimentos.isEmpty()) {
            parametros.put("GRAFICO_CENARIO_2", getGraficoPorCenario(2));
        } else {
            parametros.put("GRAFICO_CENARIO_2", null);
        }

        // Gráficos do cenário
        if (this.dadosSimulacao.isSimularCenario3() && !investimentos.isEmpty()) {
            parametros.put("GRAFICO_CENARIO_3", getGraficoPorCenario(3));
        } else {
            parametros.put("GRAFICO_CENARIO_3", null);
        }
        // Gráficos do cenário
        if (this.dadosSimulacao.isSimularCenario4() && !investimentos.isEmpty()) {
            parametros.put("GRAFICO_CENARIO_4", getGraficoPorCenario(4));
        } else {
            parametros.put("GRAFICO_CENARIO_4", null);
        }

        // Gráficos do cenário
        if (this.dadosSimulacao.isSimularCenario5() && !investimentos.isEmpty()) {
            parametros.put("GRAFICO_CENARIO_5", getGraficoPorCenario(5));
        } else {
            parametros.put("GRAFICO_CENARIO_5", null);
        }

        // Gráficos do cenário
        if (this.dadosSimulacao.isSimularCenario6() && !investimentos.isEmpty()) {
            parametros.put("GRAFICO_CENARIO_6", getGraficoPorCenario(6));
        } else {
            parametros.put("GRAFICO_CENARIO_6", null);
        }

        // Dados sobre o investimento
        parametros.put("VALOR_INVESTIDO", FuncoesDeMoedas.getValorEmFormatoMonetario(this.dadosSimulacao.getValorInvestimento()));
        parametros.put("DURACAO_INVESTIMENTO", this.dadosSimulacao.getDuracaoInvestimento().toString());
        parametros.put("INTERVALO_RELATORIO", this.dadosSimulacao.getIntervaloRelatorio().toString());
        parametros.put("DATA_APLICACAO", FuncoesDeDatas.getDataMMAAAA(this.dadosSimulacao.getMesAnoAplicacao()));
        
        jpRelatorio = JasperFillManager.fillReport(report, parametros, new JREmptyDataSource());
    }

    /**
     * ESTA FUNÇÂO DEVE SER EXECULTADA APÓS AS FUNÇÕES DE INICIALIZAÇÔES E
     * CARREGAMENTO INICIAIS DA CLASSE.
     *
     * @return
     */
    private List<ParametroStringRelatorio> getDescricaoInvestimentos() {
        List<ParametroStringRelatorio> listaDescricao = new ArrayList<>();

        for (int i = 0; i < this.investimentos.size(); i++) {
            ParametroStringRelatorio parametro = new ParametroStringRelatorio(this.investimentos.get(i).getDescricaoCompacta());
            listaDescricao.add(parametro);
        }

        return listaDescricao;
    }

    /**
     * ESTA FUNÇÂO DEVE SER EXECULTADA APÓS AS FUNÇÕES DE INICIALIZAÇÔES E
     * CARREGAMENTO INICIAIS DA CLASSE.
     *
     * @return
     */
    private List<ParametroStringRelatorio> getMensagensDeRodape() {

        boolean mensagemPrevidenciaPGBL = false;
        boolean mensagemPrevidenciaProgressiva = false;
        boolean mensagemTesouro = false;
        boolean haveInvestimentos = false;

        List<ParametroStringRelatorio> listaMensagens = new ArrayList<>();

        for (int i = 0; i < this.investimentos.size(); i++) {

            Investimento investimento = this.investimentos.get(i);

            if (investimento.isTesouroIPCA() || investimento.isTesouroSelic()) {
                mensagemTesouro = true;
            }

            if (investimento.isPrevidencia()) {
                if (((InvestimentoPrevidencia) investimento).isPGBL()) {
                    mensagemPrevidenciaPGBL = true;
                }

                if (((InvestimentoPrevidencia) investimento).isProgressiva()) {
                    mensagemPrevidenciaProgressiva = true;
                }
            }
            
            haveInvestimentos = true;
        }

        ParametroStringRelatorio parametro;

        if (mensagemTesouro) {
            parametro = new ParametroStringRelatorio("* " + InvestimentoTesouroIPCA.OBSERVACAO_QUANTO_A_MARCACAO_A_MERCADO);
            listaMensagens.add(parametro);
        }

        if (mensagemPrevidenciaPGBL) {
            parametro = new ParametroStringRelatorio("* " + InvestimentoPrevidencia.OBSERVACAO_QUANTO_A_PLANOS_PGBL);
            listaMensagens.add(parametro);
        }

        if (mensagemPrevidenciaProgressiva) {
            parametro = new ParametroStringRelatorio("* " + InvestimentoPrevidencia.OBSERVACAO_QUANTO_A_PLANOS_PROGRESSIVOS);
            listaMensagens.add(parametro);
        }
        
        String OBSERVACAO_FORMA_DE_TRATAMENTO_MES = "A simulação considera como data inicial do investimento o 1º dia do mês da aplicação" +
                                                " e a data de resgate o último dia do mês, trabalhando sempre com meses completos.";
        
        if (haveInvestimentos) {
            parametro = new ParametroStringRelatorio("* " + OBSERVACAO_FORMA_DE_TRATAMENTO_MES);
            listaMensagens.add(parametro);
        }
        

        return listaMensagens;
    }

    /**
     * ESTA FUNÇÂO DEVE SER EXECULTADA APÓS AS FUNÇÕES DE INICIALIZAÇÔES E
     * CARREGAMENTO INICIAIS DA CLASSE.
     *
     * @return List<ParametroRelatorioTabelaRentabilidade> com os dados a serem
     * exibidos na tabela RENTABILIDADE DOS INVESTIMENTOS do relatório.
     */
    private List<ParametroRelatorioTabelaRentabilidade> getDadosRelatorioTabelaRentabilidades() {

        List<ParametroRelatorioTabelaRentabilidade> dadosRelatorioTabelaRentabilidades = new ArrayList<>();

        ParametroRelatorioTabelaRentabilidade aux = null;

        Integer contIteracoesTotaisNecessarias, contInvestimentos, k;

        // calculando a quantidade de iterações necessárias para exibir os dados da rentabilidade de todo o intervalo
        // do relatório (são exibidos seis intervalos por iteração)
        int iteracoesNecessariasParaExibicaoDeTodosOsMeses = this.dadosSimulacao.getDuracaoInvestimento() / this.dadosSimulacao.getIntervaloRelatorioEmMeses() / 6;
        if (iteracoesNecessariasParaExibicaoDeTodosOsMeses * this.dadosSimulacao.getIntervaloRelatorioEmMeses() * 6 < this.dadosSimulacao.getDuracaoInvestimento()) { // caso a quantidade de intervalos a ser exibida
            // não seja multipla de da quantidade de registros
            // exibida a cada iteração realizo uma iteração a mais.
            iteracoesNecessariasParaExibicaoDeTodosOsMeses++;
        }

        // iterando sobre a quantidade de iterações necessárias para a exibição de 
        // todo o intervalo de dados do relatório (Obs: são exibidos 6 intervalos 
        // por vez.).
        for (contIteracoesTotaisNecessarias = 0; contIteracoesTotaisNecessarias < iteracoesNecessariasParaExibicaoDeTodosOsMeses; contIteracoesTotaisNecessarias++) {

            // iterando sobre a quantidade de investimentos
            for (contInvestimentos = 0; contInvestimentos < this.investimentos.size(); contInvestimentos++) {

                // inserindo linha de cabeçalho--------------------------------------------------------------
                if (contInvestimentos == 0) {
                    ParametroRelatorioTabelaRentabilidade tmp = new ParametroRelatorioTabelaRentabilidade();
                    Integer mesCabecalho = contIteracoesTotaisNecessarias * 6 * this.dadosSimulacao.getIntervaloRelatorioEmMeses() + this.dadosSimulacao.getIntervaloRelatorioEmMeses();
                    String mesA, mesB, mesC, mesD, mesE, mesF;
                    mesA = mesB = mesC = mesD = mesE = mesF = "";

                    if (mesCabecalho <= this.dadosSimulacao.getDuracaoInvestimento()) {
                        mesA = mesCabecalho.toString();
                        mesCabecalho += this.dadosSimulacao.getIntervaloRelatorioEmMeses();
                    }

                    if (mesCabecalho <= this.dadosSimulacao.getDuracaoInvestimento()) {
                        mesB = mesCabecalho.toString();
                        mesCabecalho += this.dadosSimulacao.getIntervaloRelatorioEmMeses();
                    }

                    if (mesCabecalho <= this.dadosSimulacao.getDuracaoInvestimento()) {
                        mesC = mesCabecalho.toString();
                        mesCabecalho += this.dadosSimulacao.getIntervaloRelatorioEmMeses();
                    }

                    if (mesCabecalho <= this.dadosSimulacao.getDuracaoInvestimento()) {
                        mesD = mesCabecalho.toString();
                        mesCabecalho += this.dadosSimulacao.getIntervaloRelatorioEmMeses();
                    }

                    if (mesCabecalho <= this.dadosSimulacao.getDuracaoInvestimento()) {
                        mesE = mesCabecalho.toString();
                        mesCabecalho += this.dadosSimulacao.getIntervaloRelatorioEmMeses();
                    }

                    if (mesCabecalho <= this.dadosSimulacao.getDuracaoInvestimento()) {
                        mesF = mesCabecalho.toString();
                        mesCabecalho += this.dadosSimulacao.getIntervaloRelatorioEmMeses();
                    }

                    tmp.header(mesA, mesB, mesC, mesD, mesE, mesF);
                    dadosRelatorioTabelaRentabilidades.add(tmp);
                }

                // tenho que inserir seis linhas de cenários para o investimento em questão
                Investimento investimentoAtual = this.investimentos.get(contInvestimentos);
                RentabilidadeInvestimento rentabilidadeInvestimentoAtual = this.rentabilidadeDosInvestimentos.get(contInvestimentos);

                boolean nomeDoInvestimentoJaImpresso = false;
                // imprimendo seis linhas da tabela referente ao rendimento de um
                // investimento em seis intervalos de dados (linha da tabela).
                for (Integer i = 1; i <= 6; i++) {

                    // imprime a linha apenas se o cenário em questão estiver ativo
                    if ((i == 1 && dadosSimulacao.isSimularCenario1())
                            || (i == 2 && dadosSimulacao.isSimularCenario2())
                            || (i == 3 && dadosSimulacao.isSimularCenario3())
                            || (i == 4 && dadosSimulacao.isSimularCenario4())
                            || (i == 5 && dadosSimulacao.isSimularCenario5())
                            || (i == 6 && dadosSimulacao.isSimularCenario6())) {

                        RentabilidadeInvestimento.IDENTIFICADOR_CENARIO identificador;

                        if (i == 1) {
                            identificador = RentabilidadeInvestimento.IDENTIFICADOR_CENARIO.A;
                        } else if (i == 2) {
                            identificador = RentabilidadeInvestimento.IDENTIFICADOR_CENARIO.B;
                        } else if (i == 3) {
                            identificador = RentabilidadeInvestimento.IDENTIFICADOR_CENARIO.C;
                        } else if (i == 4) {
                            identificador = RentabilidadeInvestimento.IDENTIFICADOR_CENARIO.D;
                        } else if (i == 5) {
                            identificador = RentabilidadeInvestimento.IDENTIFICADOR_CENARIO.E;
                        } else {
                            identificador = RentabilidadeInvestimento.IDENTIFICADOR_CENARIO.F;
                        }

                        int intervaloAtualmenteImpresso = contIteracoesTotaisNecessarias * 6;

                        String nomeInvestimento = "";

                        if (nomeDoInvestimentoJaImpresso == false) // imprimindo o nome do investimento caso seja a primeira linha sendo impressa.
                        {
                            nomeInvestimento = investimentoAtual.getNome();
                            nomeDoInvestimentoJaImpresso = true;
                        }

                        aux = new ParametroRelatorioTabelaRentabilidade(nomeInvestimento,
                                i.toString(),
                                rentabilidadeInvestimentoAtual.getRentabilidadeCenarioEspecifico(identificador, intervaloAtualmenteImpresso++),
                                rentabilidadeInvestimentoAtual.getRentabilidadeCenarioEspecifico(identificador, intervaloAtualmenteImpresso++),
                                rentabilidadeInvestimentoAtual.getRentabilidadeCenarioEspecifico(identificador, intervaloAtualmenteImpresso++),
                                rentabilidadeInvestimentoAtual.getRentabilidadeCenarioEspecifico(identificador, intervaloAtualmenteImpresso++),
                                rentabilidadeInvestimentoAtual.getRentabilidadeCenarioEspecifico(identificador, intervaloAtualmenteImpresso++),
                                rentabilidadeInvestimentoAtual.getRentabilidadeCenarioEspecifico(identificador, intervaloAtualmenteImpresso++));
                        dadosRelatorioTabelaRentabilidades.add(aux);
                    }
                }

            }
            // caso não seja não seja a ultima iteração insiro uma quebra de linha no relatório
            if (contIteracoesTotaisNecessarias + 1 < iteracoesNecessariasParaExibicaoDeTodosOsMeses) {
                ParametroRelatorioTabelaRentabilidade tmp = new ParametroRelatorioTabelaRentabilidade();
                tmp.quebraDelinha();
                dadosRelatorioTabelaRentabilidades.add(tmp);
            }
        }

        return dadosRelatorioTabelaRentabilidades;
    }

    /**
     * ESTA FUNÇÂO DEVE SER EXECULTADA APÓS AS FUNÇÕES DE INICIALIZAÇÔES E
     * CARREGAMENTO INICIAIS DA CLASSE.
     *
     * @return List<ParametroRelatorioTabelaIndice> com os dados a serem
     * exibidos na tabela CENÁRIOS do relatório.
     */
    private List<ParametroRelatorioTabelaIndice> getDadosRelatorioTabelaCenarios() {

        UtilRelatorioCenarios utilRelatorioCenarios = new UtilRelatorioCenarios();

        GerenciaArmazenamentoIndices gerenciaArmazenamentoIndices = new GerenciaArmazenamentoIndices();
        List<Indice> indices = gerenciaArmazenamentoIndices.getRegistros();

        return utilRelatorioCenarios.getCenarios(this.investimentos, indices);
    }

}
