/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package br.com.flaviobarros.simuladordeinvestimentos.relatorio.controller;

import br.com.flaviobarros.simuladordeinvestimentos.investimentos.indices.model.Indice;
import br.com.flaviobarros.simuladordeinvestimentos.investimentos.model.Investimento;
import br.com.flaviobarros.simuladordeinvestimentos.investimentos.model.InvestimentosComunsSemelhantes;
import br.com.flaviobarros.simuladordeinvestimentos.relatorio.model.ParametroRelatorioTabelaIndice;
import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author flavio
 */
public class UtilRelatorioCenarios {

    UtilRelatorioCenarios() {}

    // Retorna uma lista de parametros para a construção do relatório de cenários
    public List<ParametroRelatorioTabelaIndice> getCenarios(List<Investimento> investimentos, List<Indice> indices) {
        
        boolean haveSelic = false;  // indica se há investimentos atrelados a Selic cadastrados
        boolean haveCDI =false;     // indica se há investimentos atrelados ao CDI cadastrados
        boolean haveTR = false;     // indica se há investimentos atrelados a TR cadastrados
        boolean haveIPCA = false;   // indica se há investimentos atrelados ao IPCA cadastrados
        
        List<Investimento> investimentosNaoIndexados = new ArrayList<>();
        
        for (int i = 0; i < investimentos.size(); i++) {
            
            boolean isIndexado = false;
            
            // Regra para investimentos indexados ao CDI:
            // - LCI, LCA, CDB e RDB pós fixados são indexados ao CDI.
            if((investimentos.get(i).getTipo() == Investimento.TIPO.LCI ||
                investimentos.get(i).getTipo() == Investimento.TIPO.LCA ||
                investimentos.get(i).getTipo() == Investimento.TIPO.CDB ||
                investimentos.get(i).getTipo() == Investimento.TIPO.RDB)) {
                
                if (((InvestimentosComunsSemelhantes)investimentos.get(i)).getTipoDeRentabilidade() ==
                        InvestimentosComunsSemelhantes.TIPO_DE_RENTABILIDADE.POS_FIXADO){
                   
                    // o investimento em questão é indexado ao CDI
                    haveCDI = true;
                    isIndexado = true;
                    
                }  
            }
            
            // Regra para investimentos indexados ao IPCA
            // - Caso o investimento seja um Tesouro IPCA ele será indexado ao IPCA
            if (investimentos.get(i).getTipo() == Investimento.TIPO.TESOURO_IPCA){
                    haveIPCA = true;
                    isIndexado = true;
            }
            
            // Regra para investimentos indexados ao SELIC
            // - Tesouro Selic e Poupança são Indexados ao SELIC.
            if (investimentos.get(i).getTipo() == Investimento.TIPO.TESOURO_SELIC){
                    haveSelic = true;
                    isIndexado = true;
            }
            
            // Regra para investimentos indexados ao SELIC
            // - Tesouro Selic e Poupança são Indexados ao SELIC.
            // Regra para investimentos indexados a TR
            // - A poupança é indexada a TR
            if (investimentos.get(i).getTipo() == Investimento.TIPO.POUPANCA){
                    haveSelic = true;
                    haveTR = true;
                    isIndexado = true;
            }
            
            // populando Lista com investimentos indexados
            if (!isIndexado) {
                investimentosNaoIndexados.add(investimentos.get(i));
            }
        }
        
        // retorna os indices em utilização 
        List<Indice> indicesUtilizados = getIndicesUtilizados(indices, haveSelic, haveCDI, haveTR, haveIPCA);

        List<ParametroRelatorioTabelaIndice> parametros = new ArrayList<>();
        
        for (int i = 0; i < indicesUtilizados.size(); i++) {
            parametros.add(new ParametroRelatorioTabelaIndice(indicesUtilizados.get(i)));
        }
        
        for (int i = 0; i < investimentosNaoIndexados.size(); i++) {
            parametros.add(new ParametroRelatorioTabelaIndice(investimentosNaoIndexados.get(i)));
        }
        
        return parametros;
    }
    
    
    // retorna os indices em utilização de acordo com os parâmetros recebidos
    private List<Indice> getIndicesUtilizados(List<Indice> indices, boolean haveSelic, boolean haveCDI, boolean haveTR, boolean haveIPCA) {
        
        List<Indice> indicesUtilizados = new ArrayList<>();
        
        for (int i = 0; i < indices.size(); i++) {
            
            if (indices.get(i).getTipo() == Indice.TIPO.SELIC && haveSelic) {
                indicesUtilizados.add(indices.get(i));
                
            } else if (indices.get(i).getTipo() == Indice.TIPO.CDI && haveCDI) {
                indicesUtilizados.add(indices.get(i));
                
            } else if (indices.get(i).getTipo() == Indice.TIPO.TR && haveTR) {
                indicesUtilizados.add(indices.get(i));
                
            } else if (indices.get(i).getTipo() == Indice.TIPO.IPCA && haveIPCA) {
                indicesUtilizados.add(indices.get(i));
                
            } 
                
        }
        
        return indicesUtilizados;
    }
}
