/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package br.com.flaviobarros.simuladordeinvestimentos.relatorio.model;

import br.com.flaviobarros.simuladordeinvestimentos.investimentos.model.Investimento;
import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author flavio
 */
public final class RentabilidadeInvestimento {
    
    public enum IDENTIFICADOR_CENARIO{
        A,
        B,
        C,
        D,
        E,
        F        
    }
    
    /**
     * Investimento ao qual a rentabilidade se refere.
     */
    private final Investimento investimento;
    
    private List<Double> rentabilidadesCenario1;
    private List<Double> rentabilidadesCenario2;
    private List<Double> rentabilidadesCenario3;
    private List<Double> rentabilidadesCenario4;
    private List<Double> rentabilidadesCenario5;
    private List<Double> rentabilidadesCenario6;

    public RentabilidadeInvestimento(Investimento investimento) {
        this.investimento = investimento;
        
        inicializaEstrutura();
    }
    
    private void inicializaEstrutura() {
        this.rentabilidadesCenario1 = new ArrayList<>();
        this.rentabilidadesCenario2 = new ArrayList<>();
        this.rentabilidadesCenario3 = new ArrayList<>();
        this.rentabilidadesCenario4 = new ArrayList<>();
        this.rentabilidadesCenario5 = new ArrayList<>();
        this.rentabilidadesCenario6 = new ArrayList<>();
    }

    public void addRentabilidadeMes(Double rentabilidadesCenario1, Double rentabilidadesCenario2, Double rentabilidadesCenario3, Double rentabilidadesCenario4, Double rentabilidadesCenario5, Double rentabilidadesCenario6) {
        this.rentabilidadesCenario1.add(rentabilidadesCenario1);
        this.rentabilidadesCenario2.add(rentabilidadesCenario2);
        this.rentabilidadesCenario3.add(rentabilidadesCenario3);
        this.rentabilidadesCenario4.add(rentabilidadesCenario4);
        this.rentabilidadesCenario5.add(rentabilidadesCenario5);
        this.rentabilidadesCenario6.add(rentabilidadesCenario6);
    }
    
    private boolean isUmaPosicaoValida(int posicao) {
        return posicao >= 0 && posicao < rentabilidadesCenario1.size();
    }    
    
    private boolean isUmCenarioValido(int numeroCenario) {
        return numeroCenario >= 1 && numeroCenario <= 6;
    }
    
    
    /**
     * Retorna o valor da rentabilidade do Cenário solicitado para a posição recebida como parâmetro.
     * 
     * Caso a posição recebida como parâmetro seja superior inválida (menor que zero ou superior
     * a quantidade de rentabilidades calculadas) retorna null.
     * 
     * @param identificadorCenario: identificador de cenário para o qual se deseja obter a rentabilidade.
     * 
     * específica. 
     * @param posicao.............: posição da rentabilidade a ser obtida.
     * 
     * @return - valor referente a posição do cenario recebida como parâmetro caso a posição seja válida.
     *         - null, caso a posição do cenário recebida como parâmetro seja inválida.
     */
    public Double getRentabilidadeCenarioEspecifico(IDENTIFICADOR_CENARIO identificadorCenario, int posicao) {
        if (isUmaPosicaoValida(posicao)) {
            switch(identificadorCenario) {
                case A:
                    return this.rentabilidadesCenario1.get(posicao);
                case B:
                    return this.rentabilidadesCenario2.get(posicao);
                case C:
                    return this.rentabilidadesCenario3.get(posicao);
                case D:
                    return this.rentabilidadesCenario4.get(posicao);
                case E:
                    return this.rentabilidadesCenario5.get(posicao);
                case F:
                    return this.rentabilidadesCenario6.get(posicao);               
            }
        }
        
        return null;
    }
    
    /**
     * Retorna o valor da rentabilidade do Cenário 1 para a posição recebida como parâmetro.
     * Caso a posição recebida como parâmetro seja superior inválida (menor que zero ou 
     * superior a quantidade de rentabilidades calculadas) retorna null.
     * 
     * @param posicao: posição da rentabilidade a ser obtida.
     * 
     * @return o valor referente ao cenário 1 caso a posição seja válida ou null caso a
     *          posição seja inválida.
     */
    public Double getRentabilidadeCenario1(int posicao) {
        
        if (isUmaPosicaoValida(posicao)) {
            return this.rentabilidadesCenario1.get(posicao);
        } else {
            return null;
        }
        
    }
    
    public Integer getQuantidadeIntervalosCalculados() {
        return this.rentabilidadesCenario1.size();
    }
    
    /**
         * Retorna o valor da rentabilidade do Cenário 2 para a posição recebida como parâmetro.
         * Caso a posição recebida como parâmetro seja superior inválida (menor que zero ou
         * superior a quantidade de rentabilidades calculadas) retorna null.
         * 
         * @param posicao: posição da rentabilidade a ser obtida.
         * 
         * @return o valor referente ao cenário 2 caso a posição seja válida ou null caso a
         *           posição seja inválida.
         */
    public Double getRentabilidadeCenario2(int posicao) {
        
        if (isUmaPosicaoValida(posicao)) {
            return this.rentabilidadesCenario2.get(posicao);
        } else {
            return null;
        }
        
    }
    
    /**
      * Retorna o valor da rentabilidade do Cenário 3 para a posição recebida como parâmetro.
      * Caso a posição recebida como parâmetro seja superior inválida (menor que zero ou
      * superior a quantidade de rentabilidades calculadas) retorna null.
      * 
      * @param posicao: posição da rentabilidade a ser obtida.
      * 
      * @return o valor referente ao cenário 3 caso a posição seja válida ou null caso a
      *           posição seja inválida.
      */
    public Double getRentabilidadeCenario3(int posicao) {
        
        if (isUmaPosicaoValida(posicao)) {
            return this.rentabilidadesCenario3.get(posicao);
        } else {
            return null;
        }
        
    }
    
    /**
      * Retorna o valor da rentabilidade do Cenário 4 para a posição recebida como parâmetro.
      * Caso a posição recebida como parâmetro seja superior inválida (menor que zero ou
      * superior a quantidade de rentabilidades calculadas) retorna null.
      * 
      * @param posicao: posição da rentabilidade a ser obtida.
      * 
      * @return o valor referente ao cenário 4 caso a posição seja válida ou null caso a
      *           posição seja inválida.
      */
    public Double getRentabilidadeCenario4(int posicao) {
        
        if (isUmaPosicaoValida(posicao)) {
            return this.rentabilidadesCenario4.get(posicao);
        } else {
            return null;
        }
        
    }
    
    /**
      * Retorna o valor da rentabilidade do Cenário 5 para a posição recebida como parâmetro.
      * Caso a posição recebida como parâmetro seja superior inválida (menor que zero ou
      * superior a quantidade de rentabilidades calculadas) retorna null.
      * 
      * @param posicao: posição da rentabilidade a ser obtida.
      * 
      * @return o valor referente ao cenário 5 caso a posição seja válida ou null caso a
      *           posição seja inválida.
      */
    public Double getRentabilidadeCenario5(int posicao) {
        
        if (isUmaPosicaoValida(posicao)) {
            return this.rentabilidadesCenario5.get(posicao);
        } else {
            return null;
        }        
        
    }
    
    /**
      * Retorna o valor da rentabilidade do Cenário 6 para a posição recebida como parâmetro.
      * Caso a posição recebida como parâmetro seja superior inválida (menor que zero ou
      * superior a quantidade de rentabilidades calculadas) retorna null.
      * 
      * @param posicao: posição da rentabilidade a ser obtida.
      * 
      * @return o valor referente ao cenário 6 caso a posição seja válida ou null caso a
      *           posição seja inválida.
      */
    public Double getRentabilidadeCenario6(int posicao) {
        
        if (isUmaPosicaoValida(posicao)) {
            return this.rentabilidadesCenario6.get(posicao);
        } else {
            return null;
        }
        
    }
    
//--- GETTERS AND SETTERS ------------------------------------------------------

    public List<Double> getRentabilidadesCenario1() {
        return rentabilidadesCenario1;
    }

    public List<Double> getRentabilidadesCenario2() {
        return rentabilidadesCenario2;
    }

    public List<Double> getRentabilidadesCenario3() {
        return rentabilidadesCenario3;
    }

    public List<Double> getRentabilidadesCenario4() {
        return rentabilidadesCenario4;
    }

    public List<Double> getRentabilidadesCenario5() {
        return rentabilidadesCenario5;
    }

    public List<Double> getRentabilidadesCenario6() {
        return rentabilidadesCenario6;
    }

    public Investimento getInvestimento() {
        return investimento;
    }
    
    
    
    
}
