/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package br.com.flaviobarros.simuladordeinvestimentos.simulacao.view;

import br.com.flaviobarros.simuladordeinvestimentos.investimentos.controller.GerenciaArmazenamentoInvestimentos;
import br.com.flaviobarros.simuladordeinvestimentos.padroes.Padronizacoes;
import br.com.flaviobarros.simuladordeinvestimentos.relatorio.controller.GerenciaRelatorio;
import br.com.flaviobarros.simuladordeinvestimentos.simulacao.controller.ValidaSimulacao;
import br.com.flaviobarros.simuladordeinvestimentos.simulacao.model.DadosSimulacao;
import br.com.flaviobarros.util.DocumentoInteiroGenerico;
import br.com.flaviobarros.util.DocumentoMonetarioGenerico;
import java.util.Calendar;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.swing.JRViewer;

/**
 *
 * @author flavio
 */
public class SimulacaoView extends javax.swing.JPanel {

    private final Double LIMITE_VALOR_INVESTIDO = 1000000000.00;
    
    
    private final JFrame janelaMae;
    private DadosSimulacao objetoJanela;

    public SimulacaoView(JFrame janelaMae) {

        initComponents(); // método padrão de inicialização dos JavaSwing (criado automaticamente)
        inicializaComponentes(); // método de inicialização dos componentes da janela (criado manualmente)

        this.objetoJanela = new DadosSimulacao();
        this.janelaMae = janelaMae;
    }

    private void inicializaComponentes() {
        setFontes();
        inicializaCampos();
    }

    private void inicializaCampos() {

        jtfValorInvestimento.setDocument(new DocumentoMonetarioGenerico(LIMITE_VALOR_INVESTIDO, 0.0));
        jtfDuracao.setDocument(new DocumentoInteiroGenerico(360));

        inicializaComboBoxes();
        inicializaRadioButons();
        inicializaDataChoser();
        
    }
    
    private void inicializaComboBoxes() {
        //inicializando combobox IntervaloRelatorio
        jcbIntervaloRelatorio.addItem(null);
        jcbIntervaloRelatorio.addItem(DadosSimulacao.INTERVALO.MENSAL.getDescricao());
        jcbIntervaloRelatorio.addItem(DadosSimulacao.INTERVALO.BIMESTRAL.getDescricao());
        jcbIntervaloRelatorio.addItem(DadosSimulacao.INTERVALO.TRIMESTRAL.getDescricao());
        jcbIntervaloRelatorio.addItem(DadosSimulacao.INTERVALO.SEMESTRAL.getDescricao());
        jcbIntervaloRelatorio.addItem(DadosSimulacao.INTERVALO.ANUAL.getDescricao());
    }
    
    private void inicializaRadioButons(){
        
    }
    
    private void inicializaDataChoser(){
        jdcDataAplicacao.setDateFormatString("MM/yyyy");
        
        // configurando a data padrão do calendário para dia 1°
        Calendar dataPadrao = Calendar.getInstance();
        dataPadrao.set(dataPadrao.get(Calendar.YEAR), dataPadrao.get(Calendar.MONTH), 1);
        jdcDataAplicacao.setCalendar(dataPadrao);
        
        // ocultando o icone para acionamento do calendário do JDateChooser
        jdcDataAplicacao.getCalendarButton().setVisible(false);
        
    }

    private void setFontes() {
        this.jspRelatorio.setFont(Padronizacoes.getFontePadrao());
        
        this.jLabel1.setFont(Padronizacoes.getFontePadrao());
        this.jLabel2.setFont(Padronizacoes.getFontePadraoPequena());
        this.jLabel3.setFont(Padronizacoes.getFontePadrao());
        this.jLabel5.setFont(Padronizacoes.getFontePadrao());
//        this.jLabel4.setFont(Padronizacoes.getFontePadraoNegrito());
        
        this.jtfDuracao.setFont(Padronizacoes.getFontePadrao());
        this.jtfValorInvestimento.setFont(Padronizacoes.getFontePadrao());
        
        this.jcbIntervaloRelatorio.setFont(Padronizacoes.getFontePadrao());
        
        this.jrbCenario1.setFont(Padronizacoes.getFontePadrao());
        this.jrbCenario2.setFont(Padronizacoes.getFontePadrao());
        this.jrbCenario3.setFont(Padronizacoes.getFontePadrao());
        this.jrbCenario4.setFont(Padronizacoes.getFontePadrao());
        this.jrbCenario5.setFont(Padronizacoes.getFontePadrao());
        this.jrbCenario6.setFont(Padronizacoes.getFontePadrao());
        
        this.jdcDataAplicacao.setFont(Padronizacoes.getFontePadrao());
        
        this.jbGerarRelatorio.setFont(Padronizacoes.getFontePadrao());
    }

    private void carregaObjetoJanela() {
        if (this.objetoJanela == null) {
            this.objetoJanela = new DadosSimulacao();
        }

        // inicializando dados da entidade objeto da janela
        objetoJanela.setValorInvestimento(null);
        objetoJanela.setDuracaoInvestimento(null);
        objetoJanela.setIntervaloRelatorio(null);
        objetoJanela.setMesAnoAplicacao(null);

        // carregando dados da janela
        if (!jtfValorInvestimento.getText().isEmpty()) {
            this.objetoJanela.setValorInvestimento(Double.parseDouble(jtfValorInvestimento.getText()));
        }

        if (!jtfDuracao.getText().isEmpty()) {
            this.objetoJanela.setDuracaoInvestimento(Integer.parseInt(jtfDuracao.getText()));
        }
        
        this.objetoJanela.setMesAnoAplicacao(jdcDataAplicacao.getCalendar());

        if (jcbIntervaloRelatorio.getSelectedItem() == DadosSimulacao.INTERVALO.MENSAL.getDescricao()) {
            this.objetoJanela.setIntervaloRelatorio(DadosSimulacao.INTERVALO.MENSAL);
        } else if (jcbIntervaloRelatorio.getSelectedItem() == DadosSimulacao.INTERVALO.BIMESTRAL.getDescricao()) {
            this.objetoJanela.setIntervaloRelatorio(DadosSimulacao.INTERVALO.BIMESTRAL);
        } else if (jcbIntervaloRelatorio.getSelectedItem() == DadosSimulacao.INTERVALO.TRIMESTRAL.getDescricao()) {
            this.objetoJanela.setIntervaloRelatorio(DadosSimulacao.INTERVALO.TRIMESTRAL);
        } else if (jcbIntervaloRelatorio.getSelectedItem() == DadosSimulacao.INTERVALO.SEMESTRAL.getDescricao()) {
            this.objetoJanela.setIntervaloRelatorio(DadosSimulacao.INTERVALO.SEMESTRAL);
        } else if (jcbIntervaloRelatorio.getSelectedItem() == DadosSimulacao.INTERVALO.ANUAL.getDescricao()) {
            this.objetoJanela.setIntervaloRelatorio(DadosSimulacao.INTERVALO.ANUAL);
        }
        
        this.objetoJanela.setSimularCenario1(this.jrbCenario1.isSelected());
        this.objetoJanela.setSimularCenario2(this.jrbCenario2.isSelected());
        this.objetoJanela.setSimularCenario3(this.jrbCenario3.isSelected());
        this.objetoJanela.setSimularCenario4(this.jrbCenario4.isSelected());
        this.objetoJanela.setSimularCenario5(this.jrbCenario5.isSelected());
        this.objetoJanela.setSimularCenario6(this.jrbCenario6.isSelected());
        
    }

    private DadosSimulacao getObjetoJanela() {
        this.carregaObjetoJanela();
        return objetoJanela;
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jspRelatorio = new javax.swing.JScrollPane();
        jbGerarRelatorio = new javax.swing.JButton();
        jPanel4 = new javax.swing.JPanel();
        jPanel3 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        jtfValorInvestimento = new javax.swing.JTextField();
        jPanel1 = new javax.swing.JPanel();
        jrbCenario1 = new javax.swing.JRadioButton();
        jrbCenario2 = new javax.swing.JRadioButton();
        jrbCenario3 = new javax.swing.JRadioButton();
        jrbCenario4 = new javax.swing.JRadioButton();
        jrbCenario5 = new javax.swing.JRadioButton();
        jrbCenario6 = new javax.swing.JRadioButton();
        jtfDuracao = new javax.swing.JTextField();
        jdcDataAplicacao = new com.toedter.calendar.JDateChooser();
        jcbIntervaloRelatorio = new javax.swing.JComboBox<>();
        jLabel3 = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jPanel2 = new javax.swing.JPanel();
        jLabel6 = new javax.swing.JLabel();
        jLabel7 = new javax.swing.JLabel();
        jLabel8 = new javax.swing.JLabel();
        jLabel9 = new javax.swing.JLabel();
        jLabel10 = new javax.swing.JLabel();

        setPreferredSize(new java.awt.Dimension(687, 300));

        jspRelatorio.setBorder(javax.swing.BorderFactory.createTitledBorder(null, "Relatório", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("Dialog", 0, 12))); // NOI18N

        jbGerarRelatorio.setText("GERAR RELATÓRIO");
        jbGerarRelatorio.setPreferredSize(new java.awt.Dimension(150, 25));
        jbGerarRelatorio.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jbGerarRelatorioActionPerformed(evt);
            }
        });

        jPanel4.setBorder(javax.swing.BorderFactory.createTitledBorder("SIMULAÇÃO"));

        jLabel1.setText("Valor do investimento");

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder(null, "Cenários a serem utilizados", javax.swing.border.TitledBorder.DEFAULT_JUSTIFICATION, javax.swing.border.TitledBorder.DEFAULT_POSITION, new java.awt.Font("SansSerif", 0, 12))); // NOI18N

        jrbCenario1.setText("Cenário 1");

        jrbCenario2.setText("Cenário 2");

        jrbCenario3.setText("Cenário 3");

        jrbCenario4.setText("Cenário 4");

        jrbCenario5.setText("Cenário 5");

        jrbCenario6.setText("Cenário 6");

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addComponent(jrbCenario1)
                .addGap(6, 6, 6)
                .addComponent(jrbCenario2)
                .addGap(6, 6, 6)
                .addComponent(jrbCenario3)
                .addGap(6, 6, 6)
                .addComponent(jrbCenario4)
                .addGap(6, 6, 6)
                .addComponent(jrbCenario5)
                .addGap(6, 6, 6)
                .addComponent(jrbCenario6)
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jrbCenario1)
                    .addComponent(jrbCenario2)
                    .addComponent(jrbCenario3)
                    .addComponent(jrbCenario4)
                    .addComponent(jrbCenario5)
                    .addComponent(jrbCenario6))
                .addGap(0, 0, Short.MAX_VALUE))
        );

        jcbIntervaloRelatorio.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(java.awt.event.ItemEvent evt) {
                jcbIntervaloRelatorioItemStateChanged(evt);
            }
        });

        jLabel3.setText("Intervalo do Relatório ");

        jLabel5.setText("Mês/Ano aplicação");

        jLabel2.setFont(new java.awt.Font("Dialog", 0, 10)); // NOI18N
        jLabel2.setText("Prazo do investimento em meses");

        javax.swing.GroupLayout jPanel3Layout = new javax.swing.GroupLayout(jPanel3);
        jPanel3.setLayout(jPanel3Layout);
        jPanel3Layout.setHorizontalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 0, Short.MAX_VALUE)
            .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                .addGroup(jPanel3Layout.createSequentialGroup()
                    .addContainerGap()
                    .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                        .addComponent(jPanel1, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addGroup(javax.swing.GroupLayout.Alignment.LEADING, jPanel3Layout.createSequentialGroup()
                            .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                .addComponent(jtfValorInvestimento, javax.swing.GroupLayout.PREFERRED_SIZE, 155, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addComponent(jLabel1))
                            .addGap(12, 12, 12)
                            .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                .addComponent(jcbIntervaloRelatorio, javax.swing.GroupLayout.PREFERRED_SIZE, 155, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addComponent(jLabel3))
                            .addGap(12, 12, 12)
                            .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                                .addComponent(jLabel5, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addComponent(jdcDataAplicacao, javax.swing.GroupLayout.PREFERRED_SIZE, 117, javax.swing.GroupLayout.PREFERRED_SIZE))
                            .addGap(12, 12, 12)
                            .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                                .addComponent(jtfDuracao)
                                .addComponent(jLabel2))))
                    .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
        );
        jPanel3Layout.setVerticalGroup(
            jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 105, Short.MAX_VALUE)
            .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                .addGroup(jPanel3Layout.createSequentialGroup()
                    .addGap(7, 7, 7)
                    .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                        .addGroup(jPanel3Layout.createSequentialGroup()
                            .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                .addComponent(jLabel1)
                                .addComponent(jLabel3, javax.swing.GroupLayout.PREFERRED_SIZE, 15, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addComponent(jLabel2)
                                .addComponent(jLabel5))
                            .addGap(0, 0, 0)
                            .addGroup(jPanel3Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                .addComponent(jtfValorInvestimento, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addComponent(jcbIntervaloRelatorio, javax.swing.GroupLayout.PREFERRED_SIZE, 19, javax.swing.GroupLayout.PREFERRED_SIZE)))
                        .addComponent(jdcDataAplicacao, javax.swing.GroupLayout.PREFERRED_SIZE, 19, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(jtfDuracao, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGap(6, 6, 6)
                    .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
        );

        jLabel6.setFont(new java.awt.Font("Dialog", 2, 10)); // NOI18N
        jLabel6.setText("Aplicação Máxima:");

        jLabel7.setFont(new java.awt.Font("Dialog", 2, 10)); // NOI18N
        jLabel7.setText("R$ 1.000.000.000,00");

        jLabel8.setFont(new java.awt.Font("Dialog", 2, 10)); // NOI18N
        jLabel8.setText("Prazo máximo:");

        jLabel9.setFont(new java.awt.Font("Dialog", 2, 10)); // NOI18N
        jLabel9.setText("360 meses");

        jLabel10.setFont(new java.awt.Font("Dialog", 2, 10)); // NOI18N
        jLabel10.setText("Observação");

        javax.swing.GroupLayout jPanel2Layout = new javax.swing.GroupLayout(jPanel2);
        jPanel2.setLayout(jPanel2Layout);
        jPanel2Layout.setHorizontalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addGap(6, 6, 6)
                .addGroup(jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.CENTER)
                    .addComponent(jLabel10)
                    .addComponent(jLabel6)
                    .addComponent(jLabel7)
                    .addComponent(jLabel8)
                    .addComponent(jLabel9))
                .addGap(6, 6, 6))
        );
        jPanel2Layout.setVerticalGroup(
            jPanel2Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel2Layout.createSequentialGroup()
                .addComponent(jLabel10)
                .addGap(6, 6, 6)
                .addComponent(jLabel6)
                .addGap(0, 0, 0)
                .addComponent(jLabel7)
                .addGap(6, 6, 6)
                .addComponent(jLabel8)
                .addGap(0, 0, 0)
                .addComponent(jLabel9)
                .addGap(0, 9, Short.MAX_VALUE))
        );

        javax.swing.GroupLayout jPanel4Layout = new javax.swing.GroupLayout(jPanel4);
        jPanel4.setLayout(jPanel4Layout);
        jPanel4Layout.setHorizontalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel4Layout.createSequentialGroup()
                .addGap(0, 0, Short.MAX_VALUE)
                .addComponent(jPanel3, javax.swing.GroupLayout.PREFERRED_SIZE, 658, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(0, 0, 0)
                .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(0, 0, Short.MAX_VALUE))
        );
        jPanel4Layout.setVerticalGroup(
            jPanel4Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel4Layout.createSequentialGroup()
                .addComponent(jPanel3, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(0, 0, Short.MAX_VALUE))
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPanel4Layout.createSequentialGroup()
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addComponent(jPanel2, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap())
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGap(10, 10, 10)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jspRelatorio)
                    .addComponent(jbGerarRelatorio, javax.swing.GroupLayout.Alignment.CENTER, javax.swing.GroupLayout.PREFERRED_SIZE, 167, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jPanel4, javax.swing.GroupLayout.Alignment.CENTER, javax.swing.GroupLayout.DEFAULT_SIZE, 669, Short.MAX_VALUE))
                .addGap(10, 10, 10))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGap(10, 10, 10)
                .addComponent(jPanel4, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(12, 12, 12)
                .addComponent(jbGerarRelatorio, javax.swing.GroupLayout.PREFERRED_SIZE, 25, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(12, 12, 12)
                .addComponent(jspRelatorio, javax.swing.GroupLayout.DEFAULT_SIZE, 99, Short.MAX_VALUE)
                .addGap(10, 10, 10))
        );

        jspRelatorio.getAccessibleContext().setAccessibleDescription("");
    }// </editor-fold>//GEN-END:initComponents

    private void jbGerarRelatorioActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jbGerarRelatorioActionPerformed

        DadosSimulacao entidade = this.getObjetoJanela();
        
        String erro = ValidaSimulacao.valida(entidade);

        //caso todos os campos obrigatórios tenham sido preenchidos
        if (erro.isEmpty()) {
            
            GerenciaArmazenamentoInvestimentos gerenciador = new GerenciaArmazenamentoInvestimentos();
            
            // caso ainda não existam investimentos cadastrados
            if(gerenciador.getRegistrosDeTodosOsInvestimentos().isEmpty()) {
                JOptionPane.showMessageDialog(this, "Cadastre previamente os investimentos antes de realizar a simulação!");
            } else {
                try {
                    // gerando relatório
                    GerenciaRelatorio gerenciadorRelatorio = new GerenciaRelatorio(entidade);

                    // Cria um JRViewer para exibir o relatório.
                    JRViewer viewer = new JRViewer(gerenciadorRelatorio.getRelatorio());

                    // Exibe o relatório
                    this.jspRelatorio.setViewportView(viewer);

                } catch (JRException ex) {
                    JOptionPane.showMessageDialog(this, "Erro ao gerar o relatório!");
                    Logger.getLogger(SimulacaoView.class.getName()).log(Level.SEVERE, null, ex);
                } catch (Exception ex) {
                    JOptionPane.showMessageDialog(this, "Erro ao gerar o relatório!");
                    Logger.getLogger(SimulacaoView.class.getName()).log(Level.SEVERE, null, ex);
                }    
            }
      
        } else {
            JOptionPane.showMessageDialog(this, erro);
        }
    }//GEN-LAST:event_jbGerarRelatorioActionPerformed

    private void jcbIntervaloRelatorioItemStateChanged(java.awt.event.ItemEvent evt) {//GEN-FIRST:event_jcbIntervaloRelatorioItemStateChanged
        
//        jtfDuracao.setText("");
        
    }//GEN-LAST:event_jcbIntervaloRelatorioItemStateChanged


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JButton jbGerarRelatorio;
    private javax.swing.JComboBox<String> jcbIntervaloRelatorio;
    private com.toedter.calendar.JDateChooser jdcDataAplicacao;
    private javax.swing.JRadioButton jrbCenario1;
    private javax.swing.JRadioButton jrbCenario2;
    private javax.swing.JRadioButton jrbCenario3;
    private javax.swing.JRadioButton jrbCenario4;
    private javax.swing.JRadioButton jrbCenario5;
    private javax.swing.JRadioButton jrbCenario6;
    private javax.swing.JScrollPane jspRelatorio;
    private javax.swing.JTextField jtfDuracao;
    private javax.swing.JTextField jtfValorInvestimento;
    // End of variables declaration//GEN-END:variables

}
