/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package br.com.flaviobarros.util;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;

/**
 *
 * @author flavio
 */
public class DocumentoMonetarioGenerico extends PlainDocument {

    private Double valorMaximo;

    private Double valorMinimo;

    public DocumentoMonetarioGenerico() {
        this.init();
    }

    public DocumentoMonetarioGenerico(Double valorMaximo) {
        this.init();
        this.valorMaximo = valorMaximo;
    }

    public DocumentoMonetarioGenerico(Double valorMaximo, Double valorMinimo) {
        this.init();
        this.valorMaximo = valorMaximo;
        this.valorMinimo = valorMinimo;
    }

    private void init() {
        this.valorMaximo = null;
        this.valorMinimo = null;
    }

    @Override
    public void insertString(int offs, String str, AttributeSet a) throws BadLocationException {

        boolean canInsert = true;

        str = str.replaceAll("[^0|1|2|3|4|5|6|7|8|9|','|'.']", "");
        str = str.replaceAll("[,]", ".");

        // obtem o conteudo completo do campo antes da inserção da string analizado
        String strConteudo = getText(0, getLength());

        // monta uma palavra adicionando o conteudo que já consta no documento mais o conteudo a ser inserido
        strConteudo = strConteudo.substring(0, offs) + str + strConteudo.substring(offs, strConteudo.length());

        // testa a palavra que será formada caso a inserção do texto a ser inserido seja efetivada, caso ela
        // não forme um double marca o campo canInsert como false, negando a inserção.
        if (!isDouble(strConteudo)) {
            canInsert = false;
        } else {
            // testando se não há mais de duas casas decimais após a virgula.
            if (strConteudo.contains(".")) {
                int tamanho = strConteudo.length();

                if (tamanho > 2) {
                    // caso haja mais de duas casas decimais após a virgula
                    if (strConteudo.charAt(tamanho - 3) != '.' && strConteudo.charAt(tamanho - 2) != '.' && strConteudo.charAt(tamanho - 1) != '.' ) {
                        canInsert = false;
                    }
                } else if (tamanho > 1) {
                    // caso haja mais de duas casas decimais após a virgula
                    if (strConteudo.charAt(tamanho - 2) != '.' && strConteudo.charAt(tamanho - 1) != '.' ) {
                        canInsert = false;
                    }
                } else if (tamanho > 0) {
                    // caso haja mais de duas casas decimais após a virgula
                    if (strConteudo.charAt(tamanho - 1) != '.' ) {
                        canInsert = false;
                    }
                }
            }

            // caso o número a ser inserido ultrapasse o valor máximo
            if (this.valorMaximo != null) {
                if (getDouble(strConteudo) > this.valorMaximo) {
                    canInsert = false;
                }
            }

            // caso o número a ser inserido sejá inferior o valor mínimo
            if (this.valorMinimo != null) {
                if (getDouble(strConteudo) < this.valorMinimo) {
                    canInsert = false;
                }
            }
        }

        if (canInsert) {
            super.insertString(offs, str, a);
        }
    }

    private boolean isDouble(String stringValor) {

        try {

            Double number = Double.parseDouble(stringValor);

            return true;

        } catch (NumberFormatException e) {
            return false;
        }
    }

    private Double getDouble(String stringValor) {

        Double retorno = null;

        try {

            retorno = Double.parseDouble(stringValor);

            return retorno;

        } catch (NumberFormatException e) {
            return retorno;
        }
    }

}
